import unittest

from sickchill.tv import DirtySetter


class DirtySetterTests(unittest.TestCase):
    test_none = DirtySetter(None)
    test_bool = DirtySetter(False)

    def setUp(self) -> None:
        super().setUp()
        self.dirty = True

    def test_dirty_setter(self):
        assert self.dirty
        self.dirty = False

        self.assertIsNone(self.test_none)
        self.test_none = None
        assert not self.dirty

        self.test_none = 1
        assert self.dirty
        self.assertIs(self.test_none, 1)
        assert self.dirty

        self.dirty = False

        self.test_none = None
        assert self.dirty
        self.assertIsNone(self.test_none)

        assert self.dirty
        self.dirty = False

        self.test_none = dict(one=1)
        assert self.dirty
        self.assertDictEqual(self.test_none, dict(one=1))

        assert self.dirty
        self.dirty = False

        self.test_none = [dict(one=1)]
        assert self.dirty
        self.assertListEqual(self.test_none, [dict(one=1)])

        assert self.dirty
        self.dirty = False

        self.test_none = "RANDOM"
        assert self.dirty
        assert self.test_none == "RANDOM"

        assert self.dirty
        self.dirty = False

    def test_dirty_setter_stays_dirty(self):
        assert self.dirty
        self.dirty = False

        assert not self.test_bool
        self.test_bool = True
        assert self.test_bool
        assert self.dirty

        self.test_bool = False
        assert not self.test_bool
        assert self.dirty

        self.test_bool = None
        self.assertIsNone(self.test_bool)
        assert self.dirty



import collections

from .call import RPCCall


class RPCResult(object):
    def __init__(self, rpc_calls: [RPCCall], results):
        self._rpc_calls_results_map = collections.OrderedDict()
        for call, result in zip(rpc_calls, results):
            self._rpc_calls_results_map[call] = result

    def __getitem__(self, item):
        if isinstance(item, RPCCall):
            return self._rpc_calls_results_map[item]
        elif isinstance(item, int):
            key = list(self._rpc_calls_results_map.keys())[item]
            return self._rpc_calls_results_map[key]
        else:
            raise AttributeError('Received unsupported item')

    def __iter__(self):
        for key in self._rpc_calls_results_map:
            yield (key, self._rpc_calls_results_map[key])

    def __len__(self):
        return len(self._rpc_calls_results_map)



import datetime

from dateutil import tz
from tornado.web import authenticated

from sickchill import logger, settings
from sickchill.helper import try_int

from ..oldbeard import db, network_timezones
from .index import BaseHandler


class CalendarHandler(BaseHandler):
    def get(self):
        if settings.CALENDAR_UNPROTECTED:
            self.write(self.calendar())
        else:
            self.calendar_auth()

    @authenticated
    def calendar_auth(self):
        self.write(self.calendar())

    # Raw iCalendar implementation by Pedro Jose Pereira Vieito (@pvieito).
    #
    # iCalendar (iCal) - Standard RFC 5545 <http://tools.ietf.org/html/rfc5546>
    # Works with iCloud, Google Calendar and Outlook.
    def calendar(self):
        """Provides a subscribeable URL for iCal subscriptions"""

        logger.info(f"Receiving iCal request from {self.request.remote_ip}")

        # Create a iCal string
        ical = "BEGIN:VCALENDAR\r\n"
        ical += "VERSION:2.0\r\n"
        ical += "X-WR-CALNAME:SickChill\r\n"
        ical += "X-WR-CALDESC:SickChill\r\n"
        ical += "PRODID://SickChill Upcoming Episodes//\r\n"

        future_weeks = try_int(self.get_argument("future", "52"), 52)
        past_weeks = try_int(self.get_argument("past", "52"), 52)

        # Limit dates
        past_date = (datetime.date.today() + datetime.timedelta(weeks=-past_weeks)).toordinal()
        future_date = (datetime.date.today() + datetime.timedelta(weeks=future_weeks)).toordinal()

        # Get all the shows that are not paused and are currently on air (from kjoconnor Fork)
        main_db_con = db.DBConnection()
        # noinspection PyPep8
        calendar_shows = main_db_con.select(
            "SELECT show_name, indexer_id, network, airs, runtime FROM tv_shows WHERE "
            "( status = 'Continuing' OR status = 'Returning Series' ) AND paused != '1'"
        )
        for show in calendar_shows:
            # Get all episodes of this show airing between today and next month
            episode_list = main_db_con.select(
                "SELECT indexerid, name, season, episode, description, airdate FROM tv_episodes WHERE airdate >= ? AND airdate < ? AND showid = ?",
                (past_date, future_date, int(show["indexer_id"])),
            )

            utc = tz.gettz("GMT")

            for episode in episode_list:
                air_date_time = network_timezones.parse_date_time(episode["airdate"], show["airs"], show["network"]).astimezone(utc)
                air_date_time_end = air_date_time + datetime.timedelta(minutes=try_int(show["runtime"], 60))

                # Create event for episode
                ical += "BEGIN:VEVENT\r\n"
                ical += f'DTSTART:{air_date_time.strftime("%Y%m%d")}T{air_date_time.strftime("%H%M%S")}Z\r\n'
                ical += f'DTEND:{air_date_time_end.strftime("%Y%m%d")}T{air_date_time_end.strftime("%H%M%S")}Z\r\n'
                if settings.CALENDAR_ICONS:
                    ical += "X-GOOGLE-CALENDAR-CONTENT-ICON:https://sickchill.github.io/images/ico/favicon-16.png\r\n"
                    ical += "X-GOOGLE-CALENDAR-CONTENT-DISPLAY:CHIP\r\n"
                ical += f'SUMMARY: {show["show_name"]} - {episode["season"]}x{episode["episode"]} - {episode["name"]}\r\n'
                ical += f'UID:SickChill-{datetime.date.today().isoformat()}-{show["show_name"].replace(" ", "-")}-S{episode["season"]}E{episode["episode"]}\r\n'
                ical += f'DESCRIPTION:{show["airs"] or "(Unknown airs)"} on {show["network"] or "Unknown network"}'
                if episode["description"]:
                    ical += f' \\n\\n {episode["description"].splitlines()[0]}'
                ical += "\r\nEND:VEVENT\r\n"

        # Ending the iCal
        ical += "END:VCALENDAR"

        return ical



import socket

import gntp.core

from sickchill import logger, settings
from sickchill.oldbeard import common


class Notifier(object):
    def test_notify(self, host, password):
        self._sendRegistration(host, password, "Test")
        return self._sendGrowl("Test Growl", "Testing Growl settings from SickChill", "Test", host, password, force=True)

    def notify_snatch(self, ep_name):
        if settings.GROWL_NOTIFY_ONSNATCH:
            self._sendGrowl(common.notifyStrings[common.NOTIFY_SNATCH], ep_name)

    def notify_download(self, ep_name):
        if settings.GROWL_NOTIFY_ONDOWNLOAD:
            self._sendGrowl(common.notifyStrings[common.NOTIFY_DOWNLOAD], ep_name)

    def notify_subtitle_download(self, ep_name, lang):
        if settings.GROWL_NOTIFY_ONSUBTITLEDOWNLOAD:
            self._sendGrowl(common.notifyStrings[common.NOTIFY_SUBTITLE_DOWNLOAD], ep_name + ": " + lang)

    def notify_git_update(self, new_version="??"):
        update_text = common.notifyStrings[common.NOTIFY_GIT_UPDATE_TEXT]
        title = common.notifyStrings[common.NOTIFY_GIT_UPDATE]
        self._sendGrowl(title, update_text + new_version)

    def notify_login(self, ipaddress=""):
        update_text = common.notifyStrings[common.NOTIFY_LOGIN_TEXT]
        title = common.notifyStrings[common.NOTIFY_LOGIN]
        self._sendGrowl(title, update_text.format(ipaddress))

    def _send_growl(self, options, message=None):

        # Send Notification
        notice = gntp.core.GNTPNotice()

        # Required
        notice.add_header("Application-Name", options["app"])
        notice.add_header("Notification-Name", options["name"])
        notice.add_header("Notification-Title", options["title"])

        if options["password"]:
            notice.set_password(options["password"])

        # Optional
        if options["sticky"]:
            notice.add_header("Notification-Sticky", options["sticky"])
        if options["priority"]:
            notice.add_header("Notification-Priority", options["priority"])
        if options["icon"]:
            notice.add_header("Notification-Icon", settings.LOGO_URL)

        if message:
            notice.add_header("Notification-Text", message)

        response = self._send(options["host"], options["port"], notice.encode(), options["debug"])
        return True if isinstance(response, gntp.core.GNTPOK) else False

    @staticmethod
    def _send(host, port, data, debug=False):
        if debug:
            print("<Sending>\n", data, "\n</Sending>")

        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((host, port))
        s.send(data)
        response = gntp.core.parse_gntp(s.recv(1024).decode())
        s.close()

        if debug:
            print("<Received>\n", response, "\n</Received>")

        return response

    def _sendGrowl(self, title="SickChill Notification", message=None, name=None, host=None, password=None, force=False):
        if not settings.USE_GROWL and not force:
            return False

        if name is None:
            name = title

        if host is None:
            hostParts = settings.GROWL_HOST.split(":")
        else:
            hostParts = host.split(":")

        if len(hostParts) != 2 or hostParts[1] == "":
            port = 23053
        else:
            port = int(hostParts[1])

        growlHosts = [(hostParts[0], port)]

        opts = {"name": name, "title": title, "app": "SickChill", "sticky": None, "priority": None, "debug": False}

        if password is None:
            opts["password"] = settings.GROWL_PASSWORD
        else:
            opts["password"] = password

        opts["icon"] = True

        for pc in growlHosts:
            opts["host"] = pc[0]
            opts["port"] = pc[1]
            logger.debug("GROWL: Sending message '" + message + "' to " + opts["host"] + ":" + str(opts["port"]))
            try:
                if self._send_growl(opts, message):
                    return True
                else:
                    if self._sendRegistration(host, password, "SickChill"):
                        return self._send_growl(opts, message)
                    else:
                        return False
            except Exception as e:
                logger.warning("GROWL: Unable to send growl to " + opts["host"] + ":" + str(opts["port"]) + " - " + str(e))
                return False

    def _sendRegistration(self, host=None, password=None, name="SickChill Notification"):
        opts = {}

        if host is None:
            hostParts = settings.GROWL_HOST.split(":")
        else:
            hostParts = host.split(":")

        if len(hostParts) != 2 or hostParts[1] == "":
            port = 23053
        else:
            port = int(hostParts[1])

        opts["host"] = hostParts[0]
        opts["port"] = port

        if password is None:
            opts["password"] = settings.GROWL_PASSWORD
        else:
            opts["password"] = password

        opts["app"] = "SickChill"
        opts["debug"] = False

        # Send Registration
        register = gntp.core.GNTPRegister()
        register.add_header("Application-Name", opts["app"])
        register.add_header("Application-Icon", settings.LOGO_URL)

        register.add_notification("Test", True)
        register.add_notification(common.notifyStrings[common.NOTIFY_SNATCH], True)
        register.add_notification(common.notifyStrings[common.NOTIFY_DOWNLOAD], True)
        register.add_notification(common.notifyStrings[common.NOTIFY_GIT_UPDATE], True)

        if opts["password"]:
            register.set_password(opts["password"])

        try:
            return self._send(opts["host"], opts["port"], register.encode(), opts["debug"])
        except Exception as e:
            logger.warning("GROWL: Unable to send growl to " + opts["host"] + ":" + str(opts["port"]) + " - " + str(e))
            return False



# -*- coding: utf-8 -*-
import logging

from ..extensions import provider_manager, default_providers
from ..utils import hash_napiprojekt, hash_opensubtitles, hash_shooter, hash_thesubdb

logger = logging.getLogger(__name__)

hash_functions = {
    'napiprojekt': hash_napiprojekt,
    'opensubtitles': hash_opensubtitles,
    'opensubtitlesvip': hash_opensubtitles,
    'shooter': hash_shooter,
    'thesubdb': hash_thesubdb
}


def refine(video, providers=None, languages=None, **kwargs):
    """Refine a video computing required hashes for the given providers.

    The following :class:`~subliminal.video.Video` attribute can be found:

      * :attr:`~subliminal.video.Video.hashes`

    """
    if video.size <= 10485760:
        logger.warning('Size is lower than 10MB: hashes not computed')
        return

    logger.debug('Computing hashes for %r', video.name)
    for name in providers or default_providers:
        provider = provider_manager[name].plugin
        if name not in hash_functions:
            continue

        if not provider.check_types(video):
            continue

        if languages and not provider.check_languages(languages):
            continue

        video.hashes[name] = hash_functions[name](video.name)

    logger.debug('Computed hashes %r', video.hashes)



import tornado.web


class Route(object):
    routes = []

    def __init__(self, uri, name=None):
        self._uri = uri
        self.name = name

    def __call__(self, handler):
        """ Gets called when we decorate a class """
        name = self.name or handler.__name__
        self.routes.append((self._uri, handler, name))
        return handler

    @classmethod
    def get_routes(cls, web_root=""):
        cls.routes.reverse()
        routes = [tornado.web.url(web_root + _uri, handler, name=name) for _uri, handler, name, in cls.routes]
        return routes


def route_redirect(src, dst, name=None):
    Route.routes.append(tornado.web.url(src, tornado.web.RedirectHandler, dict(url=dst), name=name))



import datetime
import os
import sys
import unittest

from sickchill import tv
from sickchill.oldbeard import common
from sickchill.oldbeard.name_parser import parser
from tests import conftest

DEBUG = os.getenv("DEBUG")
VERBOSE = os.getenv("VERBOSE")

SIMPLE_TEST_CASES = {
    "standard": {
        "Mr.Show.Name.S01E02.Source.Quality.Etc-Group": parser.ParseResult(None, "Mr Show Name", 1, [2], "Source.Quality.Etc", "Group"),
        "Show.Name.S01E02": parser.ParseResult(None, "Show Name", 1, [2]),
        "Show Name - S01E02 - My Ep Name": parser.ParseResult(None, "Show Name", 1, [2], "My Ep Name"),
        "Show.1.0.Name.S01.E03.My.Ep.Name-Group": parser.ParseResult(None, "Show 1.0 Name", 1, [3], "My.Ep.Name", "Group"),
        "Show.Name.S01E02E03.Source.Quality.Etc-Group": parser.ParseResult(None, "Show Name", 1, [2, 3], "Source.Quality.Etc", "Group"),
        "Mr. Show Name - S01E02-03 - My Ep Name": parser.ParseResult(None, "Mr. Show Name", 1, [2, 3], "My Ep Name"),
        "Show.Name.S01.E02.E03": parser.ParseResult(None, "Show Name", 1, [2, 3]),
        "Show.Name-0.2010.S01E02.Source.Quality.Etc-Group": parser.ParseResult(None, "Show Name-0 2010", 1, [2], "Source.Quality.Etc", "Group"),
        "S01E02 Ep Name": parser.ParseResult(None, None, 1, [2], "Ep Name"),
        "Show Name - S06E01 - 2009-12-20 - Ep Name": parser.ParseResult(None, "Show Name", 6, [1], "2009-12-20 - Ep Name"),
        "Show Name - S06E01 - -30-": parser.ParseResult(None, "Show Name", 6, [1], "30-"),
        "Show-Name-S06E01-720p": parser.ParseResult(None, "Show-Name", 6, [1], "720p"),
        "Show-Name-S06E01-1080i": parser.ParseResult(None, "Show-Name", 6, [1], "1080i"),
        "Show.Name.S06E01.Other.WEB-DL": parser.ParseResult(None, "Show Name", 6, [1], "Other.WEB-DL"),
        "Show.Name.S06E01 Some-Stuff Here": parser.ParseResult(None, "Show Name", 6, [1], "Some-Stuff Here"),
        "Show Name - S03E14-36! 24! 36! Hut! (1)": parser.ParseResult(None, "Show Name", 3, [14], "36! 24! 36! Hut! (1)"),
    },
    "fov": {
        "Show_Name.1x02.Source_Quality_Etc-Group": parser.ParseResult(None, "Show Name", 1, [2], "Source_Quality_Etc", "Group"),
        "Show Name 1x02": parser.ParseResult(None, "Show Name", 1, [2]),
        "Show Name 1x02 x264 Test": parser.ParseResult(None, "Show Name", 1, [2], "x264 Test"),
        "Show Name - 1x02 - My Ep Name": parser.ParseResult(None, "Show Name", 1, [2], "My Ep Name"),
        "Show_Name.1x02x03x04.Source_Quality_Etc-Group": parser.ParseResult(None, "Show Name", 1, [2, 3, 4], "Source_Quality_Etc", "Group"),
        "Show Name - 1x02-03-04 - My Ep Name": parser.ParseResult(None, "Show Name", 1, [2, 3, 4], "My Ep Name"),
        "1x02 Ep Name": parser.ParseResult(None, None, 1, [2], "Ep Name"),
        "Show-Name-1x02-720p": parser.ParseResult(None, "Show-Name", 1, [2], "720p"),
        "Show-Name-1x02-1080i": parser.ParseResult(None, "Show-Name", 1, [2], "1080i"),
        "Show Name [05x12] Ep Name": parser.ParseResult(None, "Show Name", 5, [12], "Ep Name"),
        "Show.Name.1x02.WEB-DL": parser.ParseResult(None, "Show Name", 1, [2], "WEB-DL"),
    },
    "standard_repeat": {
        "Show.Name.S01E02.S01E03.Source.Quality.Etc-Group": parser.ParseResult(None, "Show Name", 1, [2, 3], "Source.Quality.Etc", "Group"),
        "Show.Name.S01E02.S01E03": parser.ParseResult(None, "Show Name", 1, [2, 3]),
        "Show Name - S01E02 - S01E03 - S01E04 - Ep Name": parser.ParseResult(None, "Show Name", 1, [2, 3, 4], "Ep Name"),
        "Show.Name.S01E02.S01E03.WEB-DL": parser.ParseResult(None, "Show Name", 1, [2, 3], "WEB-DL"),
    },
    "fov_repeat": {
        "Show.Name.1x02.1x03.Source.Quality.Etc-Group": parser.ParseResult(None, "Show Name", 1, [2, 3], "Source.Quality.Etc", "Group"),
        "Show.Name.1x02.1x03": parser.ParseResult(None, "Show Name", 1, [2, 3]),
        "Show Name - 1x02 - 1x03 - 1x04 - Ep Name": parser.ParseResult(None, "Show Name", 1, [2, 3, 4], "Ep Name"),
        "Show.Name.1x02.1x03.WEB-DL": parser.ParseResult(None, "Show Name", 1, [2, 3], "WEB-DL"),
    },
    "bare": {
        "Show.Name.102.Source.Quality.Etc-Group": parser.ParseResult(None, "Show Name", 1, [2], "Source.Quality.Etc", "Group"),
        "show.name.2010.123.source.quality.etc-group": parser.ParseResult(None, "show name 2010", 1, [23], "source.quality.etc", "group"),
        "show.name.2010.222.123.source.quality.etc-group": parser.ParseResult(None, "show name 2010.222", 1, [23], "source.quality.etc", "group"),
        "Show.Name.102": parser.ParseResult(None, "Show Name", 1, [2]),
        "Show.Name.01e02": parser.ParseResult(None, "Show Name", 1, [2]),
        "the.event.401.hdtv-group": parser.ParseResult(None, "the event", 4, [1], "hdtv", "group"),
        "show.name.2010.special.hdtv-blah": None,
        "show.ex-name.102.hdtv-group": parser.ParseResult(None, "show ex-name", 1, [2], "hdtv", "group"),
    },
    "stupid": {"tpz-abc102": parser.ParseResult(None, "abc", 1, [2], None, "tpz"), "tpz-abc.102": parser.ParseResult(None, "abc", 1, [2], None, "tpz")},
    "no_season": {
        "Show Name - 01 - Ep Name": parser.ParseResult(None, "Show Name", None, [1], "Ep Name"),
        "01 - Ep Name": parser.ParseResult(None, None, None, [1], "Ep Name"),
        "Show Name - 01 - Ep Name - WEB-DL": parser.ParseResult(None, "Show Name", None, [1], "Ep Name - WEB-DL"),
    },
    "no_season_general": {
        "Show.Name.E23.Source.Quality.Etc-Group": parser.ParseResult(None, "Show Name", None, [23], "Source.Quality.Etc", "Group"),
        "Show Name - Episode 01 - Ep Name": parser.ParseResult(None, "Show Name", None, [1], "Ep Name"),
        "Show.Name.Part.3.Source.Quality.Etc-Group": parser.ParseResult(None, "Show Name", None, [3], "Source.Quality.Etc", "Group"),
        "Show.Name.Part.1.and.Part.2.Blah-Group": parser.ParseResult(None, "Show Name", None, [1, 2], "Blah", "Group"),
        "Show.Name.Part.IV.Source.Quality.Etc-Group": parser.ParseResult(None, "Show Name", None, [4], "Source.Quality.Etc", "Group"),
        "Deconstructed.E07.1080i.HDTV.DD5.1.MPEG2-TrollHD": parser.ParseResult(None, "Deconstructed", None, [7], "1080i.HDTV.DD5.1.MPEG2", "TrollHD"),
        "Show.Name.E23.WEB-DL": parser.ParseResult(None, "Show Name", None, [23], "WEB-DL"),
    },
    "no_season_multi_ep": {
        "Show.Name.E23-24.Source.Quality.Etc-Group": parser.ParseResult(None, "Show Name", None, [23, 24], "Source.Quality.Etc", "Group"),
        "Show Name - Episode 01-02 - Ep Name": parser.ParseResult(None, "Show Name", None, [1, 2], "Ep Name"),
        "Show.Name.E23-24.WEB-DL": parser.ParseResult(None, "Show Name", None, [23, 24], "WEB-DL"),
    },
    "season_only": {
        "Show.Name.S02.Source.Quality.Etc-Group": parser.ParseResult(None, "Show Name", 2, [], "Source.Quality.Etc", "Group"),
        "Show Name Season 2": parser.ParseResult(None, "Show Name", 2),
        "Season 02": parser.ParseResult(None, None, 2),
    },
    "scene_date_format": {
        "Show.Name.2010.11.23.Source.Quality.Etc-Group": parser.ParseResult(
            None, "Show Name", None, [], "Source.Quality.Etc", "Group", datetime.date(2010, 11, 23)
        ),
        "Show Name - 2010.11.23": parser.ParseResult(None, "Show Name", air_date=datetime.date(2010, 11, 23)),
        "Show.Name.2010.23.11.Source.Quality.Etc-Group": parser.ParseResult(
            None, "Show Name", None, [], "Source.Quality.Etc", "Group", datetime.date(2010, 11, 23)
        ),
        "Show Name - 2010-11-23 - Ep Name": parser.ParseResult(None, "Show Name", extra_info="Ep Name", air_date=datetime.date(2010, 11, 23)),
        "2010-11-23 - Ep Name": parser.ParseResult(None, extra_info="Ep Name", air_date=datetime.date(2010, 11, 23)),
        "Show.Name.2010.11.23.WEB-DL": parser.ParseResult(None, "Show Name", None, [], "WEB-DL", None, datetime.date(2010, 11, 23)),
    },
}

ANIME_TEST_CASES = {
    "anime_SxxExx": {
        "Show Name - S01E02 - Ep Name": parser.ParseResult(None, "Show Name", 1, [2], "Ep Name"),
        "Show Name - S01E02-03 - My Ep Name": parser.ParseResult(None, "Show Name", 1, [2, 3]),
        "Show Name - S01E02": parser.ParseResult(None, "Show Name", 1, [2]),
        "Show Name - S01E02-03": parser.ParseResult(None, "Show Name", 1, [2, 3]),
    },
    "anime_bare": {
        "Show Name - 102": parser.ParseResult(None, "Show Name", ab_episode_numbers=[102], version=1),
        "[SC]_Show_Name_123": parser.ParseResult(None, "Show Name", release_group="SC", ab_episode_numbers=[123], version=1),
        "Show.Name.-.134.-.1080p.BluRay.x264.DHD": parser.ParseResult(
            None, "Show Name", ab_episode_numbers=[134], quality=common.Quality.FULLHDBLURAY, version=1
        ),
    },
}

COMBINATION_TEST_CASES = [
    ("/test/path/to/Season 02/03 - Ep Name.avi", parser.ParseResult(None, None, 2, [3], "Ep Name"), ["no_season", "season_only"]),
    (
        "Show.Name.S02.Source.Quality.Etc-Group/tpz-sn203.avi",
        parser.ParseResult(None, "Show Name", 2, [3], "Source.Quality.Etc", "Group"),
        ["stupid", "season_only"],
    ),
    ("MythBusters.S08E16.720p.HDTV.x264-aAF/aaf-mb.s08e16.720p.mkv", parser.ParseResult(None, "MythBusters", 8, [16], "720p.HDTV.x264", "aAF"), ["standard"]),
    (
        "/home/drop/storage/TV/Terminator The Sarah Connor Chronicles/Season 2/S02E06 The Tower is Tall, But the Fall is Short.mkv",
        parser.ParseResult(None, None, 2, [6], "The Tower is Tall, But the Fall is Short"),
        ["standard"],
    ),
    (
        r"/Test/TV/Jimmy Fallon/Season 2/Jimmy Fallon - 2010-12-15 - blah.avi",
        parser.ParseResult(None, "Jimmy Fallon", extra_info="blah", air_date=datetime.date(2010, 12, 15)),
        ["scene_date_format"],
    ),
    (r"/X/30 Rock/Season 4/30 Rock - 4x22 -.avi", parser.ParseResult(None, "30 Rock", 4, [22]), ["fov"]),
    ("Season 2\\Show Name - 03-04 - Ep Name.ext", parser.ParseResult(None, "Show Name", 2, [3, 4], extra_info="Ep Name"), ["no_season", "season_only"]),
    ("Season 02\\03-04-05 - Ep Name.ext", parser.ParseResult(None, None, 2, [3, 4, 5], extra_info="Ep Name"), ["no_season", "season_only"]),
]

UNICODE_TEST_CASES = [
    (
        "The.Big.Bang.Theory.2x07.The.Panty.Piñata.Polarization.720p.HDTV.x264.AC3-SHELDON.mkv",
        parser.ParseResult(None, "The.Big.Bang.Theory", 2, [7], "The.Panty.Piñata.Polarization.720p.HDTV.x264.AC3", "SHELDON"),
    ),
    (
        "The.Big.Bang.Theory.2x07.The.Panty.Piñata.Polarization.720p.HDTV.x264.AC3-SHELDON.mkv",
        parser.ParseResult(None, "The.Big.Bang.Theory", 2, [7], "The.Panty.Piñata.Polarization.720p.HDTV.x264.AC3", "SHELDON"),
    ),
]

FAILURE_CASES = ["7sins-jfcs01e09-720p-bluray-x264"]


class UnicodeTests(conftest.SickChillTestDBCase):
    """
    Test str
    """

    def __init__(self, something):
        super().__init__(something)
        super().setUp()
        self.show = tv.TVShow(1, 1, "en")
        self.show.name = "The Big Bang Theory"

    def _test_unicode(self, name, result):
        """
        Test str

        :param name:
        :param result:
        :return:
        """
        name_parser = parser.NameParser(True, showObj=self.show)
        parse_result = name_parser.parse(name)

        # this shouldn't raise an exception
        repr(str(parse_result))
        assert parse_result.extra_info == result.extra_info

    def test_unicode(self):
        """
        Test str
        """
        for (name, result) in UNICODE_TEST_CASES:
            self._test_unicode(name, result)


class FailureCaseTests(conftest.SickChillTestDBCase):
    """
    Test cases that should fail
    """

    @staticmethod
    def _test_name(name):
        """
        Test name

        :param name:
        :return:
        """
        name_parser = parser.NameParser(True)
        try:
            parse_result = name_parser.parse(name)
        except (parser.InvalidNameException, parser.InvalidShowException):
            return True

        if VERBOSE:
            print("Actual: ", parse_result.which_regex, parse_result)
        return False

    def test_failures(self):
        """
        Test failures
        """
        for name in FAILURE_CASES:
            assert self._test_name(name)


class ComboTests(conftest.SickChillTestDBCase):
    """
    Perform combination tests
    """

    def _test_combo(self, name, result, which_regexes):
        """
        Perform combination test

        :param name:
        :param result:
        :param which_regexes:
        :return:
        """

        if VERBOSE:
            print()
            print("Testing", name)

        name_parser = parser.NameParser(True)

        try:
            test_result = name_parser.parse(name)
        except parser.InvalidShowException:
            return False

        if DEBUG:
            print(test_result, test_result.which_regex)
            print(result, which_regexes)

        assert str(test_result) == str(result)
        for cur_regex in which_regexes:
            assert cur_regex in test_result.which_regex
        assert len(which_regexes) == len(test_result.which_regex)

    def test_combos(self):
        """
        Perform combination tests
        """
        for (name, result, which_regexes) in COMBINATION_TEST_CASES:
            # Normalise the paths. Converts UNIX-style paths into Windows-style
            # paths when test is run on Windows.
            self._test_combo(os.path.normpath(name), result, which_regexes)


class BasicTests(conftest.SickChillTestDBCase):
    """
    Basic name parsing tests
    """

    def __init__(self, something):
        super().__init__(something)
        super().setUp()
        self.show = tv.TVShow(1, 1, "en")

    def _test_names(self, name_parser, section, transform=None, verbose=False):
        """
        Performs a test

        :param name_parser: to use for test
        :param section:
        :param transform:
        :param verbose:
        :return:
        """

        if VERBOSE or verbose:
            print()
            print("Running", section, "tests")
        for cur_test_base in SIMPLE_TEST_CASES[section]:
            if transform:
                cur_test = transform(cur_test_base)
                name_parser.filename = cur_test
            else:
                cur_test = cur_test_base
            if VERBOSE or verbose:
                print("Testing", cur_test)

            result = SIMPLE_TEST_CASES[section][cur_test_base]

            self.show.name = result.series_name if result else None
            name_parser.showObj = self.show
            if not result:
                self.assertRaises(parser.InvalidNameException, name_parser.parse, cur_test)
                return
            else:
                result.which_regex = [section]
                test_result = name_parser.parse(cur_test)

            def print_debug():
                print(f"{cur_test}:")
                print(f"Test Result: {test_result}")
                print(f"Expected Result: {result}")

            if DEBUG or verbose:
                print_debug()

            result.score = test_result.score  # Needed so we don't have to specify expected regex score in each test case.
            assert test_result.which_regex == [section], print_debug()
            assert str(test_result) == str(result), print_debug()

    def test_standard_names(self):
        """
        Test standard names
        """
        name_parser = parser.NameParser(True)
        self._test_names(name_parser, "standard")

    def test_standard_filenames(self):
        """
        Test standard file names
        """
        name_parser = parser.NameParser()
        self._test_names(name_parser, "standard", lambda x: x + ".avi")

    def test_standard_repeat_names(self):
        """
        Test standard repeat names
        """
        name_parser = parser.NameParser(False)
        self._test_names(name_parser, "standard_repeat")

    def test_standard_repeat_filenames(self):
        """
        Test standard repeat file names
        """
        name_parser = parser.NameParser()
        self._test_names(name_parser, "standard_repeat", lambda x: x + ".avi")

    def test_fov_names(self):
        """
        Test fov names
        """
        name_parser = parser.NameParser(False)
        self._test_names(name_parser, "fov")

    def test_fov_filenames(self):
        """
        Test fov file names
        """
        name_parser = parser.NameParser()
        self._test_names(name_parser, "fov", lambda x: x + ".avi")

    def test_fov_repeat_names(self):
        """
        Test fov repeat names
        """
        name_parser = parser.NameParser(False)
        self._test_names(name_parser, "fov_repeat")

    def test_fov_repeat_filenames(self):
        """
        Test fov repeat file names
        """
        name_parser = parser.NameParser()
        self._test_names(name_parser, "fov_repeat", lambda x: x + ".avi")

    def test_stupid_names(self):
        """
        Test stupid names
        """
        name_parser = parser.NameParser(False)
        self._test_names(name_parser, "stupid")

    def test_stupid_filenames(self):
        """
        Test stupid file names
        """
        name_parser = parser.NameParser()
        self._test_names(name_parser, "stupid", lambda x: x + ".avi")

    def test_no_s_general_names(self):
        """
        Test no season general names
        """
        name_parser = parser.NameParser(False)
        self._test_names(name_parser, "no_season_general")

    def test_no_s_general_filenames(self):
        """
        Test no season general file names
        """
        name_parser = parser.NameParser()
        self._test_names(name_parser, "no_season_general", lambda x: x + ".avi")

    def test_no_s_multi_ep_names(self):
        """
        Test no season multi episode names
        """
        name_parser = parser.NameParser(False)
        self._test_names(name_parser, "no_season_multi_ep")

    def test_no_s_multi_ep_filenames(self):
        """
        Test no season multi episode file names
        """
        name_parser = parser.NameParser()
        self._test_names(name_parser, "no_season_multi_ep", lambda x: x + ".avi")

    def test_s_only_names(self):
        """
        Test season only names
        """
        name_parser = parser.NameParser(False)
        self._test_names(name_parser, "season_only")

    def test_s_only_filenames(self):
        """
        Test season only file names
        """
        name_parser = parser.NameParser()
        self._test_names(name_parser, "season_only", lambda x: x + ".avi")


class AnimeTests(conftest.SickChillTestDBCase):
    """
    Basic tests for anime
    """

    def __init__(self, something):
        super().__init__(something)
        super().setUp()
        self.show = tv.TVShow(1, 1, "en")
        self.show.anime = True

    def tearDown(self):
        parser.name_parser_cache.data.clear()

    def _test_names(self, name_parser, section, transform=None, verbose=False):
        """
        Performs a test

        :param name_parser: to use for test
        :param section:
        :param transform:
        :param verbose:
        :return:
        """
        if VERBOSE or verbose:
            print()
            print("Running", section, "tests")
        for cur_test_base in ANIME_TEST_CASES[section]:
            if transform:
                cur_test = transform(cur_test_base)
                name_parser.filename = cur_test
            else:
                cur_test = cur_test_base
            if VERBOSE or verbose:
                print("Testing", cur_test)

            result = ANIME_TEST_CASES[section][cur_test_base]

            self.show.name = result.series_name if result else None
            name_parser.showObj = self.show
            if not result:
                self.assertRaises(parser.InvalidNameException, name_parser.parse, cur_test)
                return
            else:
                result.which_regex = [section]
                test_result = name_parser.parse(cur_test)

            def print_debug():
                print(f"{cur_test}:")
                print(f"Test Result: {test_result}")
                print(f"Expected Result: {result}")

            if DEBUG or verbose:
                print_debug()

            result.score = test_result.score  # Needed so we don't have to specify expected regex score in each test case.
            assert test_result.which_regex == [section], print_debug()
            assert str(test_result) == str(result), print_debug()

    def test_anime_sxxexx_filenames(self):
        """
        Test anime SxxExx file names
        """
        name_parser = parser.NameParser(parse_method="anime")
        self._test_names(name_parser, "anime_SxxExx", lambda x: x + ".avi")

    def test_anime_bare_filenames(self):
        """
        Test anime bare file names
        """
        name_parser = parser.NameParser(parse_method="anime")
        self._test_names(name_parser, "anime_bare", lambda x: x + ".avi")

    @unittest.expectedFailure
    def test_anime_bare_filenames_indirect(self):
        """
        Test anime bare file names without defining parse method
        """
        name_parser = parser.NameParser()
        self._test_names(name_parser, "anime_bare", lambda x: x + ".avi")


# TODO: Make these work or document why they shouldn't
class BasicFailedTests(conftest.SickChillTestDBCase):
    """
    Basic tests that currently fail
    """

    def __init__(self, something):
        super().__init__(something)
        super().setUp()
        self.show = tv.TVShow(1, 1, "en")

    def _test_names(self, name_parser, section, transform=None, verbose=False):
        """
        Performs a test

        :param name_parser: to use for test
        :param section:
        :param transform:
        :param verbose:
        :return:
        """
        if VERBOSE or verbose:
            print()
            print("Running", section, "tests")
        for cur_test_base in SIMPLE_TEST_CASES[section]:
            if transform:
                cur_test = transform(cur_test_base)
                name_parser.filename = cur_test
            else:
                cur_test = cur_test_base
            if VERBOSE or verbose:
                print("Testing", cur_test)

            result = SIMPLE_TEST_CASES[section][cur_test_base]

            self.show.name = result.series_name if result else None
            name_parser.showObj = self.show
            if not result:
                self.assertRaises(parser.InvalidNameException, name_parser.parse, cur_test)
                return
            else:
                result.which_regex = [section]
                test_result = name_parser.parse(cur_test)

            def print_debug():
                print(f"{cur_test}:")
                print(f"Test Result: {test_result}")
                print(f"Expected Result: {result}")

            if DEBUG or verbose:
                print_debug()

            result.score = test_result.score  # Needed so we don't have to specify expected regex score in each test case.
            assert test_result.which_regex == [section], print_debug()
            assert str(test_result) == str(result), print_debug()

    def test_no_s_names(self):
        """
        Test no season names
        """
        name_parser = parser.NameParser(False)
        self._test_names(name_parser, "no_season")

    def test_no_s_filenames(self):
        """
        Test no season file names
        """
        name_parser = parser.NameParser()
        self._test_names(name_parser, "no_season", lambda x: x + ".avi")

    @unittest.expectedFailure
    def test_bare_names(self):
        """
        Test bare names
        """
        name_parser = parser.NameParser(False)
        self._test_names(name_parser, "bare")

    @unittest.expectedFailure
    def test_bare_filenames(self):
        """
        Test bare file names
        """
        name_parser = parser.NameParser()
        self._test_names(name_parser, "bare", lambda x: x + ".avi")

    @unittest.skip("Not yet implemented")
    def test_combination_names(self):
        """
        Test combination names
        """
        pass

    @unittest.skip("Not trying indexer")
    def test_scene_date_fmt_names(self):
        """
        Test scene date format names
        """
        name_parser = parser.NameParser(False)
        self._test_names(name_parser, "scene_date_format")

    @unittest.skip("Not trying indexer")
    def test_scene_date_fmt_filenames(self):
        """
        Test scene date format file names
        """
        name_parser = parser.NameParser()
        self._test_names(name_parser, "scene_date_format", lambda x: x + ".avi")


if __name__ == "__main__":
    if len(sys.argv) > 1:
        SUITE = unittest.TestLoader().loadTestsFromName("name_parser_tests.BasicTests.test_" + sys.argv[1])
        unittest.TextTestRunner(verbosity=2).run(SUITE)
    else:
        SUITE = unittest.TestLoader().loadTestsFromTestCase(BasicTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(ComboTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(UnicodeTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(FailureCaseTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(AnimeTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)



# -*- coding: utf-8 -*-

"""
This module implements the base class of tvdbsimple.

Handle automatically login, token creation and response basic stripping.

[See Authentication API section](https://api.thetvdb.com/swagger#!/Authentication)
"""

import json
import requests


class AuthenticationError(Exception):
    """
    Authentication exception class for authentication errors
    """
    pass

class APIKeyError(Exception):
    """
    Missing API key exception class in case of missing api
    """
    pass

class TVDB(object):
    """
    Basic Authentication class for API key, login and token automatic handling functionality.

    [See Authentication API section](https://api.thetvdb.com/swagger#!/Authentication)
    """
    _headers = {'Content-Type': 'application/json',
               'Accept': 'application/json',
               'Connection': 'close'}
    _BASE_PATH = ''
    _URLS = {}
    _BASE_URI = 'https://api.thetvdb.com'

    def __init__(self, id=0, user=None, key=None):
        """
        Initialize the base class.
        
        You can provide `id` that is the item id used for url creation. You can also 
        provide `user`, that is the username for login. 
        You can also provide `key`, that is the userkey needed to 
        authenticate with the user, you can find it in the 
        [account info](http://thetvdb.com/?tab=userinfo) under account identifier., 
        the language id you want to use to retrieve the info.
        """
        self._ID = id
        self.USER = user
        """Stores username if available"""
        self.USER_KEY = key
        """Stores user-key if available"""

    def _get_path(self, key):
        return self._BASE_PATH + self._URLS[key]

    def _get_id_path(self, key):
        return self._get_path(key).format(id=self._ID)

    def _get_complete_url(self, path):
        return '{base_uri}/{path}'.format(base_uri=self._BASE_URI, path=path)

    def _set_language(self, language):
        if language:
            self._headers['Accept-Language'] = language
    
    def refresh_token(self):
        """
        Refresh the current token set in the module.

        Returns the new obtained valid token for the API.
        """
        self._set_token_header()

        response = requests.request(
            'GET', self._get_complete_url('refresh_token'),
            headers=self._headers)

        response.raise_for_status()
        jsn = response.json()
        if 'token' in jsn:
            from . import KEYS
            KEYS.API_TOKEN = jsn['token']
            return KEYS.API_TOKEN
        return ''

    def _set_token_header(self, forceNew=False):
        self._headers['Authorization'] = 'Bearer ' + self.get_token(forceNew)

    def get_token(self, forceNew=False):
        """
        Get the existing token or creates it if it doesn't exist.
        Returns the API token.

        If `forceNew` is true  the function will do a new login to retrieve the token.
        """
        from . import KEYS
        if not KEYS.API_TOKEN or forceNew:
            if not KEYS.API_KEY:
                raise APIKeyError

            if hasattr(self,"USER") and hasattr(self,"USER_KEY"):
                data = {"apikey": KEYS.API_KEY, "username": self.USER, "userkey": self.USER_KEY}
            else:
                data={"apikey": KEYS.API_KEY}

            response = requests.request(
                    'POST', self._get_complete_url('login'), 
                    data=json.dumps(data), 
                    headers=self._headers)
            if response.status_code == 200:
                KEYS.API_TOKEN = response.json()['token']
            else:
                error = "Unknown error while authenticating. Check your api key or your user/userkey"
                try:
                    error = response.json()['Error']
                except:
                    pass
                raise AuthenticationError(error)
        return KEYS.API_TOKEN

    def _request(self, method, path, params=None, payload=None, forceNewToken=False, cleanJson = True):
        self._set_token_header(forceNewToken)
        
        url = self._get_complete_url(path)

        response = requests.request(
            method, url, params=params, 
            data=json.dumps(payload) if payload else payload,
            headers=self._headers)
        
        if response.status_code == 200:
            response.encoding = 'utf-8'
            jsn = response.json()
            if cleanJson and 'data' in jsn:
                return jsn['data']
            return jsn
        elif not forceNewToken:
            return self._request(method=method, path=path, params=params, payload=payload, forceNewToken=True)
        try:
            raise Exception(response.json()['Error'])
        except:
            response.raise_for_status()

    def _GET(self, path, params=None, cleanJson = True):
        return self._request('GET', path, params=params, cleanJson=cleanJson)

    def _POST(self, path, params=None, payload=None, cleanJson = True):
        return self._request('POST', path, params=params, payload=payload, cleanJson=cleanJson)

    def _DELETE(self, path, params=None, payload=None, cleanJson = True):
        return self._request('DELETE', path, params=params, payload=payload, cleanJson=cleanJson)

    def _PUT(self, path, params=None, payload=None, cleanJson = True):
        return self._request('PUT', path, params=params, payload=payload, cleanJson=cleanJson)

    def _set_attrs_to_values(self, response={}):
        """
        Set attributes to dictionary values.

        - e.g.
        >>> import tvdbsimple as tvdb
        >>> show = tmdb.Tv(10332)
        >>> response = show.info()
        >>> show.title  # instead of response['title']
        """
        if isinstance(response, dict):
            for key in response:
                setattr(self, key, response[key])


# Copyright 2004-2017 Davide Alberani <da@erlug.linux.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This package provides various parsers to access IMDb data, such as
a parser for the web/http interface, a parser for the SQL database interface,
etc. So far, the http, s3 and sql parsers are implemented.
"""

from __future__ import absolute_import, division, print_function, unicode_literals


__all__ = ['http', 'sql', 's3']



import os
import time

from tornado.web import addslash

from sickchill import settings
from sickchill.oldbeard import helpers
from sickchill.views.common import PageTemplate
from sickchill.views.routes import Route

from .index import Config


@Route("/config/backuprestore(/?.*)", name="config:backup")
class ConfigBackupRestore(Config):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @addslash
    def index(self, *args_, **kwargs_):
        t = PageTemplate(rh=self, filename="config_backuprestore.mako")

        return t.render(
            submenu=self.ConfigMenu(),
            title=_("Config - Backup/Restore"),
            header=_("Backup/Restore"),
            topmenu="config",
            controller="config",
            action="backupRestore",
        )

    @staticmethod
    def backup(backupDir=None):

        finalResult = ""

        if backupDir:
            source = [
                os.path.join(settings.DATA_DIR, "sickchill.db"),
                settings.CONFIG_FILE,
                os.path.join(settings.DATA_DIR, "failed.db"),
                os.path.join(settings.DATA_DIR, "cache.db"),
            ]
            target = os.path.join(backupDir, "sickchill-" + time.strftime("%Y%m%d%H%M%S") + ".zip")

            for (path, dirs, files) in os.walk(settings.CACHE_DIR, topdown=True):
                for dirname in dirs:
                    if path == settings.CACHE_DIR and dirname not in ["images"]:
                        dirs.remove(dirname)
                for filename in files:
                    source.append(os.path.join(path, filename))

            if helpers.backup_config_zip(source, target, settings.DATA_DIR):
                finalResult += "Successful backup to " + target
            else:
                finalResult += "Backup FAILED"
        else:
            finalResult += "You need to choose a folder to save your backup to!"

        finalResult += "<br>\n"

        return finalResult

    @staticmethod
    def restore(backupFile=None):

        finalResult = ""

        if backupFile:
            source = backupFile
            target_dir = os.path.join(settings.DATA_DIR, "restore")

            if helpers.restore_config_zip(source, target_dir):
                finalResult += "Successfully extracted restore files to " + target_dir
                finalResult += "<br>Restart sickchill to complete the restore."
            else:
                finalResult += "Restore FAILED"
        else:
            finalResult += "You need to select a backup file to restore!"

        finalResult += "<br>\n"

        return finalResult



import re
from collections import OrderedDict
from urllib.parse import urljoin

from sickchill import settings
from sickchill.oldbeard import helpers
from sickchill.oldbeard.clients.generic import GenericClient


class Client(GenericClient):
    def __init__(self, host=None, username=None, password=None):
        """
        Initializes the utorrent client class and sets the url, username, and password
        """
        super().__init__("uTorrent", host, username, password)
        self.url = urljoin(self.host, "gui/")

    def _request(self, method="get", params=None, data=None, files=None, cookies=None):
        """
        Overrides the parent _request method to add the auth token
        """
        ordered_params = OrderedDict({"token": self.auth})
        for k, v in params.items() or {}:
            ordered_params.update({k: v})

        return super()._request(method=method, params=ordered_params, data=data, files=files, cookies=cookies)

    def _get_auth(self):
        """
        Makes a request to the token url to get a CSRF token
        """
        try:
            self.response = self.session.get(urljoin(self.url, "token.html"), verify=False)
            self.response.raise_for_status()
            self.auth = re.findall("<div.*?>(.*?)</", self.response.text)[0]
        except Exception as error:
            helpers.handle_requests_exception(error)
            self.auth = None

        return self.auth

    def _add_torrent_uri(self, result):
        """
        Adds a torrent either by magnet or url
        params: :result: an instance of the searchResult class
        """
        params = {"action": "add-url", "s": result.url}
        return self._request(params=params)

    def _add_torrent_file(self, result):
        """
        Adds a torrent file from memory
        params: :result: an instance of the searchResult class
        """
        params = {"action": "add-file"}
        files = {"torrent_file": (result.name + ".torrent", result.content)}
        return self._request(method="post", params=params, files=files)

    def _set_torrent_label(self, result):
        """
        Sets a label on an existing torrent in the client
        params: :result: an instance of the searchResult class
        """
        label = (settings.TORRENT_LABEL, settings.TORRENT_LABEL_ANIME)[result.show.is_anime] or settings.TORRENT_LABEL
        params = {"action": "setprops", "hash": result.hash, "s": "label", "v": label}
        return self._request(params=params)

    def _set_torrent_ratio(self, result):
        """
        Sets the desired seed ratio for an existing torrent in the client
        params: :result: an instance of the searchResult class
        """
        if result.ratio in (None, ""):
            return True

        params = {"action": "setprops", "hash": result.hash, "s": "seed_override", "v": "1"}
        if not self._request(params=params):
            return False

        params = {"action": "setprops", "hash": result.hash, "s": "seed_ratio", "v": float(result.ratio) * 10}
        return self._request(params=params)

    def _set_torrent_seed_time(self, result):
        """
        Sets the amount of time a torrent that exists in the client should seed for
        params: :result: an instance of the searchResult class
        """
        if not settings.TORRENT_SEED_TIME:
            return True

        params = {"action": "setprops", "hash": result.hash, "s": "seed_override", "v": "1"}

        if not self._request(params=params):
            return False

        params = {"action": "setprops", "hash": result.hash, "s": "seed_time", "v": 3600 * float(settings.TORRENT_SEED_TIME)}
        return self._request(params=params)

    def _set_torrent_priority(self, result):
        """
        Sets the priority of a torrent that exists in the client
        params: :result: an instance of the searchResult class
        """
        if not result.priority:
            return True

        params = {"action": "queuetop", "hash": result.hash}
        return self._request(params=params)

    def _set_torrent_pause(self, result):
        """
        Pauses a torrent that exists on the client
        params: :result: an instance of the searchResult class
        """
        params = {"action": "pause" if settings.TORRENT_PAUSED else "start", "hash": result.hash}
        return self._request(params=params)



# # from anidbhttp import anidbquery
# from anidbhttp.query import QUERY_HOT
#
# # from sickchill.oldbeard import helpers
# from sickchill.helper.common import try_int
#
# # from .recommended import RecommendedShow
#
#
# class AnidbPopular(object):
#     def __init__(self):
#         self.cache_subfolder = __name__.split('.')[-1] if '.' in __name__ else __name__
#         self.session = helpers.make_session()
#
#     def fetch_latest_hot_shows(self):
#         """Get popular show information from IMDB"""
#
#         result = []
#
#         shows = anidbquery.query(QUERY_HOT)
#         for show in shows:
#             try:
#                 recommended_show = RecommendedShow(show.id, show.titles['x-jat'][0], 1, show.tvdbid, cache_subfolder=self.cache_subfolder,
#                      rating=str(show.ratings['temporary']['rating']), votes=str(try_int(show.ratings['temporary']['count'], 0)), image_href=show.url)
#
#                 # Check cache or get and save image
#                 recommended_show.cache_image("http://img7.anidb.net/pics/anime/{0}".format(show.image_path))
#
#                 result.append(recommended_show)
#             except Exception:
#                 pass
#
#         return result
#
# anidb_popular = AnidbPopular()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2016-9-12

@author: drummersbrother
'''

LOCALE = [
    ["just nu", "ett tag"],
    ["för %s sekunder sedan", "om %s sekunder"],
    ["för 1 minut sedan", "om 1 minut"],
    ["för %s minuter sedan", "om %s minuter"],
    ["för 1 timme sedan", "om 1 timme"],
    ["för %s timmar sedan", "om %s timmar"],
    ["för 1 dag sedan", "om 1 dag"],
    ["för %s dagar sedan", "om %s dagar"],
    ["för 1 vecka sedan", "om 1 vecka"],
    ["för %s veckor sedan", "om %s veckor"],
    ["för 1 månad sedan", "om 1 månad"],
    ["för %s månader sedan", "om %s månader"],
    ["för 1 år sedan", "om 1 år"],
    ["för %s år sedan", "om %s år"],
]



import xmlrpc.client

from .call import RPCCall
from .method import RPCMethod
from .result import RPCResult


class RPCCaller(object):
    def __init__(self, context):
        self.context = context
        self.calls = []
        self.available_methods = None

    def add(self, *args):
        if isinstance(args[0], RPCCall):
            call = args[0]
        elif isinstance(args[0], RPCMethod):
            call = RPCCall(args[0], *args[1:])
        elif isinstance(args[0], str):
            call = RPCCall(RPCMethod(args[0]), *args[1:])
        elif hasattr(args[0], '__self__'):
            call = args[0].__self__.rpc_call(args[0].__name__, *args[1:])
        else:
            raise RuntimeError("Unexpected args[0]: {0}".format(args[0]))

        self.calls.append(call)
        return self

    def call(self):
        multi_call = xmlrpc.client.MultiCall(self.context.get_conn())
        for rpc_call in self.calls:
            method_name = self._get_method_name(rpc_call.get_method())
            rpc_call.do_pre_processing()
            getattr(multi_call, method_name)(*rpc_call.get_args())

        results = []
        for rpc_call, result in zip(self.calls, multi_call()):
            print(rpc_call.get_method().get_method_names())
            result = rpc_call.do_post_processing(result)
            results.append(result)

        return RPCResult(self.calls, results)

    def _get_method_name(self, rpc_method: RPCMethod):
        if self.available_methods is None:
            self.available_methods = self.context.get_available_rpc_methods()

        method_name = rpc_method.get_available_method_name(
            self.available_methods)

        if method_name is None:
            # TODO: Use a different Error subclass
            raise AttributeError("No matches found for {0}".format(
                rpc_method.get_method_names()))

        return method_name



"""
Test Provider Result Parsing
When recording new cassettes:
    Delete the cassette yml file with the same base filename as this file in the cassettes dir next to this file
    Be sure to adjust the self.search_strings so they return results. They must be identical to search strings generated by SickChill
"""

import re
import sys
import unittest
from functools import wraps

import pytest
import validators

import sickchill.oldbeard.providers
from sickchill import movies, settings

settings.CPU_PRESET = "NORMAL"


disabled_provider_tests = {
    # ???
    "Cpasbien": ["test_rss_search", "test_episode_search", "test_season_search"],
    "SkyTorrents": ["test_rss_search", "test_episode_search", "test_season_search"],
    "ilCorsaroNero": ["test_rss_search"],
    # api_maintenance still
    "TorrentProject": ["test_rss_search", "test_episode_search", "test_season_search", "test_cache_update", "test_result_values"],
    # Have to trick it into thinking is an anime search, and add string overrides
    "TokyoToshokan": ["test_rss_search", "test_episode_search", "test_season_search"],
    "LimeTorrents": ["test_rss_search", "test_episode_search", "test_season_search"],
    "Torrentz": ["test_rss_search", "test_episode_search", "test_season_search", "test_cache_update", "test_result_values"],
    "ThePirateBay": ["test_rss_search", "test_episode_search", "test_season_search", "test_cache_update", "test_result_values"],
    "EZTV": ["test_rss_search", "test_episode_search", "test_season_search", "test_cache_update", "test_result_values"],
    "Rarbg": ["test_season_search", "test_episode_search", "test_rss_search"],
    # Demonoid is onion only now
    "Demonoid": ["test_rss_search", "test_episode_search", "test_season_search", "test_cache_update", "test_result_values"],
    # HorribleSubs needs rewritten
    "HorribleSubs": ["test_rss_search", "test_episode_search", "test_season_search", "test_cache_update", "test_result_values"],
}

test_string_overrides = {
    "Cpasbien": {"Episode": ["The 100 S07E08"], "Season": ["The 100 S06"]},
    "Torrent9": {"Episode": ["The 100 S07E08"], "Season": ["The 100 S06"]},
    "Nyaa": {"Episode": ["Fairy Tail S2"], "Season": ["Fairy Tail S2"]},
    "TokyoToshokan": {"Episode": ["Fairy Tail S2"], "Season": ["Fairy Tail S2"]},
    "HorribleSubs": {"Episode": ["Fairy Tail S2"], "Season": ["Fairy Tail S2"]},
    "Demonoid": {"Episode": ["Star Trek Picard S01E04"], "Season": ["Locke and Key 2020 S01"]},
    "ilCorsaroNero": {"Episode": ["The 100 S06E02"]},
    "Rarbg": {"Season": ["Person of Interest S02"], "Episode": ["WandaVision S01E09"]},
}

magnet_regex = re.compile(r"magnet:\?xt=urn:btih:\w{32,40}(:?&dn=[\w. %+-]+)*(:?&tr=(:?tcp|https?|udp)[\w%. +-]+)*")


class BaseParser(type):
    class TestCase(unittest.TestCase):
        provider = None

        def __init__(self, test):
            """Initialize the test suite"""
            super().__init__(test)

            self.provider.session.verify = True
            settings.SSL_VERIFY = True

            settings.QUALITY_ALLOW_HEVC = True
            self.provider.username = self.username
            self.provider.password = self.password
            settings.movie_list = movies.MovieList()

        @property
        def username(self):
            # TODO: Make this read usernames from somewhere
            return ""

        @property
        def password(self):
            # TODO: Make this read passwords from somewhere
            return ""

        def search_strings(self, mode):
            _search_strings = {"RSS": [""], "Episode": ["The 100 S07E08"], "Season": ["Game of Thrones S08"], "Movie": ["Black Panther 2018"]}
            _search_strings.update(self.provider.cache.search_params)
            _search_strings.update(test_string_overrides.get(self.provider.name, {}))
            return {mode: _search_strings[mode]}

        def magic_skip(func):  # pylint:disable=no-self-argument
            @wraps(func)
            def magic(self, *args, **kwargs):
                if func.__name__ in disabled_provider_tests.get(self.provider.name, []):
                    self.skipTest("Test is programmatically disabled for provider {}".format(self.provider.name))
                func(self, *args, **kwargs)

            return magic

        def shortDescription(self):
            if self._testMethodDoc:
                return self._testMethodDoc.replace("the provider", self.provider.name)
            return None

        @magic_skip
        @pytest.mark.vcr()
        def test_rss_search(self):
            """Check that the provider parses rss search results"""
            if self.provider.can_daily:
                results = self.provider.search(self.search_strings("RSS"))
                assert results, results

        @magic_skip
        @pytest.mark.vcr()
        def test_episode_search(self):
            """Check that the provider parses episode search results"""
            if self.provider.can_backlog:
                results = self.provider.search(self.search_strings("Episode"))
                assert results, results

        @magic_skip
        @pytest.mark.vcr()
        def test_season_search(self):
            """Check that the provider parses season search results"""
            if self.provider.can_backlog:
                results = self.provider.search(self.search_strings("Season"))
                assert results, results

        @magic_skip
        @pytest.mark.vcr()
        def test_movie_search(self):
            """Check that the provider parses episode search results"""
            if self.provider.supports_movies:
                results = self.provider.search(self.search_strings("Movie"))
                assert results, results

        @magic_skip
        @pytest.mark.vcr()
        def test_cache_update(self):
            """Check that the provider's cache parses rss search results"""
            if self.provider.can_daily:
                self.provider.cache.update_cache()

        @magic_skip
        @pytest.mark.vcr()
        def test_result_values(self):
            """Check that the provider returns results in proper format"""
            if self.provider.can_backlog:
                for result in self.provider.search(self.search_strings("Episode")):
                    self.assertIsInstance(result, dict)
                    assert sorted(result) == ["hash", "leechers", "link", "seeders", "size", "title"]

                    self.assertIsInstance(result["title"], str)
                    self.assertIsInstance(result["link"], str)
                    self.assertIsInstance(result["hash"], str)
                    self.assertIsInstance(result["seeders"], int)
                    self.assertIsInstance(result["leechers"], int)
                    self.assertIsInstance(result["size"], int)

                    assert len(result["title"])
                    assert len(result["link"])
                    assert len(result["hash"]) in (0, 32, 40)
                    assert result["seeders"] >= 0
                    assert result["leechers"] >= 0

                    assert result["size"] >= -1

                    if result["link"].startswith("magnet"):
                        assert magnet_regex.match(result["link"])
                    else:
                        assert validators.url(result["link"]), result["link"]

                    self.assertIsInstance(self.provider._get_size(result), int)
                    assert all(self.provider._get_title_and_url(result))
                    assert self.provider._get_size(result)

            @pytest.skip("Not yet implemented")
            def test_season_search_strings_format(self):
                """Check format of the provider's season search strings"""
                pass

            @pytest.skip("Not yet implemented")
            def test_episode_search_strings_format(self):
                """Check format of the provider's season search strings"""
                pass


def generate_test_cases():
    """
    Auto Generate TestCases from providers and add them to globals()
    """
    for p in sickchill.oldbeard.providers.__all__:
        provider = sickchill.oldbeard.providers.getProviderModule(p).Provider()
        if provider.can_backlog and provider.provider_type == "torrent" and provider.public:
            generated_class = type(str(provider.name), (BaseParser.TestCase,), {"provider": provider})
            globals()[generated_class.__name__] = generated_class
            del generated_class


generate_test_cases()

if __name__ == "__main__":
    import inspect

    print("=====> Testing %s", __file__)

    def override_log(msg, *args, **kwargs):
        """Override the SickChill logger so we can see the debug output from providers"""
        _ = args, kwargs
        print(msg)

    sickchill.logger.info = override_log
    sickchill.logger.debug = override_log
    sickchill.logger.error = override_log
    sickchill.logger.warning = override_log

    suite = unittest.TestSuite()
    members = inspect.getmembers(sys.modules[__name__], inspect.isclass)
    for _, provider_test_class in members:
        if provider_test_class not in (BaseParser, BaseParser.TestCase):
            suite.addTest(unittest.TestLoader().loadTestsFromTestCase(provider_test_class))

    unittest.TextTestRunner(verbosity=3).run(suite)



"""
Episode tagger to extract information from episodes
"""
import re

from sickchill.helper.common import try_int
from sickchill.recompiled import tags


class EpisodeTags(object):
    """
    Quality tags
    """

    def __init__(self, name):
        self.name = name
        self.rex = {
            "res": tags.resolution,
            "bluray": tags.bluray,
            "web": tags.web,
            "itunes": tags.itunes,
            "dvd": tags.dvd,
            "sat": tags.sat,
            "tv": tags.tv,
            "avc": tags.avc,
            "mpeg": tags.mpeg,
            "xvid": tags.xvid,
            "netflix": tags.netflix,
            "amazon": tags.amazon,
        }

    def _get_match_obj(self, attr, regex=None, flags=0):
        match_obj = "{0}_match".format(attr)
        try:
            return getattr(self, match_obj)
        except (KeyError, AttributeError):
            regexes = regex or self.rex[attr]
            if not isinstance(regexes, list):
                regexes = [regexes]
            for regexItem in regexes:
                result = regexItem.search(self.name, flags)
                if result:
                    break
            setattr(self, match_obj, result)
            return result

    # RESOLUTION
    @property
    def res(self):
        """
        The resolution tag found in the name

        :returns: an empty string if not found
        """
        attr = "res"
        match = self._get_match_obj(attr)
        return "" if not match else match.group().lower()

    @property
    def vres(self):
        """
        The vertical found in the name

        :returns: an empty string if not found
        """
        attr = "res"
        match = self._get_match_obj(attr)
        return None if not match else try_int(match.group("vres"))

    @property
    def scan(self):
        """
        The type of scan found in the name

        e.g. `i` for Interlaced, `p` for Progressive Scan

        :returns: an empty string if not found
        """
        attr = "res"
        match = self._get_match_obj(attr)
        return match.group("scan").lower() if match and match.group("scan") else ""

    # SOURCES
    @property
    def bluray(self):
        """
        The bluray tag found in the name

        :returns: an empty string if not found
        """
        attr = "bluray"
        match = self._get_match_obj(attr)
        return "" if not match else match.group()

    @property
    def hddvd(self):
        """
        The hddvd tag found in the name

        :returns: an empty string if not found
        """
        attr = "dvd"
        match = self._get_match_obj(attr)
        return None if not match else match.group("hd")

    @property
    def itunes(self):
        """
        The iTunes tag found in the name

        :returns: an empty string if not found
        """
        attr = "itunes"
        match = self._get_match_obj(attr)
        return "" if not match else match.group()

    @property
    def web(self):
        """
        The web tag found in the name

        :returns: an empty string if not found
        """
        if "dlmux" in self.name.lower():
            return "dlmux"
        if self.netflix:
            return self.netflix
        else:
            attr = "web"
            match = self._get_match_obj(attr)
            return "" if not match else match.group("type") or "dl"

    @property
    def sat(self):
        """
        The sat tag found in the name

        :returns: an empty string if not found
        """
        attr = "sat"
        match = self._get_match_obj(attr)
        return None if not match else match.group()

    @property
    def dvdrip(self):
        """
        The dvd tag found in the name

        :returns: an empty string if not found
        """
        attr = "dvd"
        match = self._get_match_obj(attr)
        return "" if not match else match.group("rip")

    @property
    def dvd(self):
        """
        The dvd tag found in the name

        :returns: an empty string if not found
        """
        attr = "dvd"
        match = self._get_match_obj(attr)
        return "" if not (match or self.hddvd) else match.group()

    @property
    def tv(self):
        attr = "tv"
        match = self._get_match_obj(attr)
        return "" if not match else (match.group(1) or match.group(2)).lower()

    # CODECS
    @property
    def hevc(self):
        """
        The hevc tag found in the name

        :returns: an empty string if not found
        """
        if not self.avc:
            return ""
        return "" if not (self.avc[-1] == "5") else self.avc

    @property
    def avc(self):
        """
        The avc tag found in the name

        :returns: an empty string if not found
        """
        attr = "avc"
        match = self._get_match_obj(attr)
        return "" if not match else match.group()

    @property
    def avc_free(self):
        """
        The free avc codec found in the name
        e.g.: x.265 or X264

        :returns: an empty string if not found
        """
        return "" if not self.avc.lower().startswith("x") else self.avc

    @property
    def avc_non_free(self):
        """
        The non-free avc codec found in the name
        e.g.: h.265 or H264

        :returns: an empty string if not found
        """
        return "" if not self.avc.lower().startswith("h") else self.avc

    @property
    def mpeg(self):
        """
        The mpeg tag found in the name

        :returns: an empty string if not found
        """
        attr = "mpeg"
        match = self._get_match_obj(attr)
        return "" if not match else match.group()

    @property
    def xvid(self):
        """
        The xvid tag found in the name

        :returns: an empty string if not found
        """
        attr = "xvid"
        match = self._get_match_obj(attr)
        return "" if not match else match.group()

    # MISCELLANEOUS
    @property
    def hrws(self):
        """
        The hrws tag found in the name

        HR = High Resolution
        WS = Wide Screen
        PD TV = Pure Digital Television

        :returns: an empty string if not found
        """
        attr = "hrws"
        match = None
        if self.avc and self.tv == "pd":
            regex = re.compile(r"(hr.ws.pdtv).{0}".format(self.avc), re.I)
            match = self._get_match_obj(attr, regex)
        return "" if not match else match.group()

    @property
    def raw(self):
        """
        The raw tag found in the name

        :return: an empty string if not found
        """
        attr = "raw"
        match = None
        if self.res and self.tv == "hd":
            regex = re.compile(r"({0}.hdtv)".format(self.res), re.I)
            match = self._get_match_obj(attr, regex)
        if not match:
            regex = re.compile(r"(RawHD)", re.I)
            match = self._get_match_obj(attr, regex)
        return "" if not match else match.group()

    @property
    def netflix(self):
        """
        Netflix tage found in name
        :return: an empty string if not found
        """
        attr = "netflix"
        match = self._get_match_obj(attr)
        return "" if not match else match.group()

    @property
    def amazon(self):
        """
        Amazon tage found in name
        :return: an empty string if not found
        """
        attr = "amazon"
        match = self._get_match_obj(attr)
        return "" if not match else match.group()

    def __str__(self):
        # TODO: Add other class properties into this output
        out = list()
        out.append(self.name)
        out.extend("{}: {}".format(attr, getattr(self, attr)) for attr in self.rex if getattr(self, attr))
        return "\n".join(out)



import requests
import fanart
from fanart.errors import RequestFanartError, ResponseFanartError


class Request(object):
    def __init__(self, apikey, id, ws, type=None, sort=None, limit=None):
        '''
        .. warning:: Since the migration to fanart.tv's api v3, we cannot use
            the kwargs `type/sort/limit` as we did before, so for now this
            kwargs will be ignored.
        '''
        self._apikey = apikey
        self._id = id
        self._ws = ws
        self._type = type or fanart.TYPE.ALL
        self._sort = sort or fanart.SORT.POPULAR
        self._limit = limit or fanart.LIMIT.ALL
        self.validate()
        self._response = None

    def validate(self):
        for attribute_name in ('ws', 'type', 'sort', 'limit'):
            attribute = getattr(self, '_' + attribute_name)
            choices = getattr(fanart, attribute_name.upper() + '_LIST')
            if attribute not in choices:
                raise RequestFanartError(
                    'Not allowed {}: {} [{}]'.format(
                        attribute_name, attribute, ', '.join(choices)))

    def __str__(self):
        return '{base_url}/{ws}/{id}?api_key={apikey}'.format(
            base_url=fanart.BASEURL,
            ws=self._ws,
            id=self._id,
            apikey=self._apikey,
        )

    def response(self):
        try:
            response = requests.get(str(self))
            rjson = response.json()
            if not isinstance(rjson, dict):
                raise Exception(response.text)
            if 'error message' in rjson:
                raise Exception(rjson['status'], rjson['error message'])
            return rjson
        except Exception as e:
            raise ResponseFanartError(str(e))



import datetime
import operator
import re
import threading
import time
import traceback

from sickchill import logger, oldbeard, settings
from sickchill.helper.exceptions import AuthException
from sickchill.show.History import History

from . import db, helpers
from .common import cpu_presets, DOWNLOADED, Quality, SNATCHED, SNATCHED_PROPER
from .name_parser.parser import InvalidNameException, InvalidShowException, NameParser
from .search import pickBestResult, snatchEpisode


class ProperFinder(object):
    def __init__(self):
        self.amActive = False

    def run(self, force=False):
        """
        Start looking for new propers

        :param force: Start even if already running (currently not used, defaults to False)
        """
        logger.info("Beginning the search for new propers")

        self.amActive = True

        propers = self._getProperList()

        if propers:
            self._downloadPropers(propers)

        self._set_lastProperSearch(datetime.datetime.today().toordinal())

        run_at = ""
        if settings.properFinderScheduler.start_time is None:
            run_in = settings.properFinderScheduler.lastRun + settings.properFinderScheduler.cycleTime - datetime.datetime.now()
            hours, remainder = divmod(run_in.seconds, 3600)
            minutes, seconds = divmod(remainder, 60)
            run_at = ", next check in approx. " + ("{0:d}h, {1:d}m".format(hours, minutes) if hours > 0 else "{0:d}m, {1:d}s".format(minutes, seconds))

        logger.info("Completed the search for new propers{0}".format(run_at))

        self.amActive = False

    def _getProperList(self):
        """
        Walk providers for propers
        """
        propers = {}

        search_date = datetime.datetime.today() - datetime.timedelta(days=2)

        # for each provider get a list of the
        origThreadName = threading.currentThread().name
        providers = [x for x in oldbeard.providers.sortedProviderList(settings.RANDOMIZE_PROVIDERS) if x.is_active]
        for curProvider in providers:
            threading.currentThread().name = origThreadName + " :: [" + curProvider.name + "]"

            logger.info("Searching for any new PROPER releases from " + curProvider.name)

            try:
                curPropers = curProvider.find_propers(search_date)
            except AuthException as e:
                logger.warning("Authentication error: " + str(e))
                continue
            except Exception as e:
                logger.exception("Exception while searching propers in " + curProvider.name + ", skipping: " + str(e))
                logger.debug(traceback.format_exc())
                continue

            # if they haven't been added by a different provider than add the proper to the list
            for x in curPropers:
                if not re.search(r"\b(proper|repack|real)\b", x.name, re.I):
                    logger.debug("find_propers returned a non-proper, we have caught and skipped it.")
                    continue

                name = self._genericName(x.name)
                if name not in propers:
                    logger.debug("Found new proper: " + x.name)
                    x.provider = curProvider
                    propers[name] = x

            threading.currentThread().name = origThreadName

        # take the list of unique propers and get it sorted by
        sortedPropers = sorted(list(propers.values()), key=operator.attrgetter("date"), reverse=True)
        finalPropers = []

        for curProper in sortedPropers:

            try:
                parse_result = NameParser(False).parse(curProper.name)
            except (InvalidNameException, InvalidShowException) as error:
                logger.debug("{0}".format(error))
                continue

            if not parse_result.series_name:
                continue

            if parse_result.show.paused:
                logger.debug("Ignoring " + curProper.name + " because " + parse_result.show.name + " is paused")
                continue

            if not parse_result.episode_numbers:
                logger.debug("Ignoring " + curProper.name + " because it's for a full season rather than specific episode")
                continue

            logger.debug("Successful match! Result " + parse_result.original_name + " matched to show " + parse_result.show.name)

            # set the indexerid in the db to the show's indexerid
            curProper.indexerid = parse_result.show.indexerid

            # set the indexer in the db to the show's indexer
            curProper.indexer = parse_result.show.indexer

            # populate our Proper instance
            curProper.show = parse_result.show
            curProper.season = parse_result.season_number if parse_result.season_number is not None else 1
            curProper.episode = parse_result.episode_numbers[0]
            curProper.release_group = parse_result.release_group
            curProper.version = parse_result.version
            curProper.quality = Quality.nameQuality(curProper.name, parse_result.is_anime)
            curProper.content = None

            # filter release
            bestResult = pickBestResult(curProper, parse_result.show)
            if not bestResult:
                logger.debug("Proper " + curProper.name + " were rejected by our release filters.")
                continue

            # only get anime proper if it has release group and version
            if bestResult.show.is_anime and not bestResult.release_group and bestResult.version == -1:
                logger.debug("Proper " + bestResult.name + " doesn't have a release group and version, ignoring it")
                continue

            # check if we actually want this proper (if it's the right quality)
            main_db_con = db.DBConnection()
            sql_results = main_db_con.select(
                "SELECT status FROM tv_episodes WHERE showid = ? AND season = ? AND episode = ?", [bestResult.indexerid, bestResult.season, bestResult.episode]
            )
            if not sql_results:
                continue

            # only keep the proper if we have already retrieved the same quality ep (don't get better/worse ones)
            oldStatus, oldQuality = Quality.splitCompositeStatus(int(sql_results[0]["status"]))
            if oldStatus not in (DOWNLOADED, SNATCHED) or oldQuality != bestResult.quality:
                continue

            # check if we actually want this proper (if it's the right release group and a higher version)
            if bestResult.show.is_anime:
                main_db_con = db.DBConnection()
                sql_results = main_db_con.select(
                    "SELECT release_group, version FROM tv_episodes WHERE showid = ? AND season = ? AND episode = ?",
                    [bestResult.indexerid, bestResult.season, bestResult.episode],
                )

                oldVersion = int(sql_results[0]["version"])
                oldRelease_group = sql_results[0]["release_group"]

                if -1 < oldVersion < bestResult.version:
                    logger.info("Found new anime v" + str(bestResult.version) + " to replace existing v" + str(oldVersion))
                else:
                    continue

                if oldRelease_group != bestResult.release_group:
                    logger.info(
                        "Skipping proper from release group: " + bestResult.release_group + ", does not match existing release group: " + oldRelease_group
                    )
                    continue

            # if the show is in our list and there hasn't been a proper already added for that particular episode then add it to our list of propers
            if bestResult.indexerid != -1 and (bestResult.indexerid, bestResult.season, bestResult.episode) not in {
                (p.indexerid, p.season, p.episode) for p in finalPropers
            }:
                logger.info("Found a proper that we need: " + str(bestResult.name))
                finalPropers.append(bestResult)

        return finalPropers

    def _downloadPropers(self, properList):
        """
        Download proper (snatch it)

        :param properList:
        """

        for curProper in properList:
            historyLimit = datetime.datetime.today() - datetime.timedelta(days=30)

            # make sure the episode has been downloaded before
            main_db_con = db.DBConnection()
            historyResults = main_db_con.select(
                "SELECT resource FROM history "
                + "WHERE showid = ? AND season = ? AND episode = ? AND quality = ? AND date >= ? "
                + "AND action IN ("
                + ",".join([str(x) for x in Quality.SNATCHED + Quality.DOWNLOADED])
                + ")",
                [curProper.indexerid, curProper.season, curProper.episode, curProper.quality, historyLimit.strftime(History.date_format)],
            )

            # if we didn't download this episode in the first place we don't know what quality to use for the proper so we can't do it
            if not historyResults:
                logger.info("Unable to find an original history entry for proper " + curProper.name + " so I'm not downloading it.")
                continue

            else:

                # make sure that none of the existing history downloads are the same proper we're trying to download
                clean_proper_name = self._genericName(helpers.remove_non_release_groups(curProper.name))
                isSame = False
                for curResult in historyResults:
                    # if the result exists in history already we need to skip it
                    if self._genericName(helpers.remove_non_release_groups(curResult["resource"])) == clean_proper_name:
                        isSame = True
                        break
                if isSame:
                    logger.debug("This proper is already in history, skipping it")
                    continue

                # get the episode object
                epObj = curProper.show.getEpisode(curProper.season, curProper.episode)

                # make the result object
                result = curProper.provider.get_result([epObj])
                result.show = curProper.show
                result.url = curProper.url
                result.name = curProper.name
                result.quality = curProper.quality
                result.release_group = curProper.release_group
                result.version = curProper.version
                result.content = curProper.content

                # snatch it
                snatchEpisode(result, SNATCHED_PROPER)
                time.sleep(cpu_presets[settings.CPU_PRESET])

    @staticmethod
    def _genericName(name):
        return name.replace(".", " ").replace("-", " ").replace("_", " ").lower()

    @staticmethod
    def _set_lastProperSearch(when):
        """
        Record last propersearch in DB

        :param when: When was the last proper search
        """

        logger.debug("Setting the last Proper search in the DB to " + str(when))

        main_db_con = db.DBConnection()
        sql_results = main_db_con.select("SELECT last_proper_search FROM info")

        if not sql_results:
            main_db_con.action("INSERT INTO info (last_backlog, last_indexer, last_proper_search) VALUES (?,?,?)", [0, 0, str(when)])
        else:
            main_db_con.action("UPDATE info SET last_proper_search=" + str(when))

    @staticmethod
    def _get_lastProperSearch():
        """
        Find last propersearch from DB
        """

        main_db_con = db.DBConnection()
        sql_results = main_db_con.select("SELECT last_proper_search FROM info")

        try:
            last_proper_search = datetime.date.fromordinal(int(sql_results[0]["last_proper_search"]))
        except Exception:
            return datetime.date.min

        return last_proper_search



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2017-8-30

@author: generated by @lolobosse script
'''
LOCALE = [
    ["זה עתה", "עכשיו"],
    ["לפני %s שניות", "בעוד %s שניות"],
    ["לפני דקה", "בעוד דקה"],
    ["לפני %s דקות", "בעוד %s דקות"],
    ["לפני שעה", "בעוד שעה"],
    ["לפני %s שעות", "בעוד %s שעות"],
    ["אתמול", "מחר"],
    ["לפני %s ימים", "בעוד %s ימים"],
    ["לפני שבוע", "בעוד שבוע"],
    ["לפני %s שבועות", "בעוד %s שבועות"],
    ["לפני חודש", "בעוד חודש"],
    ["לפני %s חודשים", "בעוד %s חודשים"],
    ["לפני שנה", "בעוד שנה"],
    ["לפני %s שנים", "בעוד %s שנים"]
]
        


# -*- coding: utf-8 -*-
from babelfish import LanguageReverseConverter, language_converters


class Addic7edConverter(LanguageReverseConverter):
    def __init__(self):
        self.name_converter = language_converters['name']
        self.from_addic7ed = {u'Català': ('cat',), 'Chinese (Simplified)': ('zho',), 'Chinese (Traditional)': ('zho',),
                              'Euskera': ('eus',), 'Galego': ('glg',), 'Greek': ('ell',), 'Malay': ('msa',),
                              'Portuguese (Brazilian)': ('por', 'BR'), 'Serbian (Cyrillic)': ('srp', None, 'Cyrl'),
                              'Serbian (Latin)': ('srp',), 'Spanish (Latin America)': ('spa',),
                              'Spanish (Spain)': ('spa',)}
        self.to_addic7ed = {('cat',): 'Català', ('zho',): 'Chinese (Simplified)', ('eus',): 'Euskera',
                            ('glg',): 'Galego', ('ell',): 'Greek', ('msa',): 'Malay',
                            ('por', 'BR'): 'Portuguese (Brazilian)', ('srp', None, 'Cyrl'): 'Serbian (Cyrillic)'}
        self.codes = self.name_converter.codes | set(self.from_addic7ed.keys())

    def convert(self, alpha3, country=None, script=None):
        if (alpha3, country, script) in self.to_addic7ed:
            return self.to_addic7ed[(alpha3, country, script)]
        if (alpha3, country) in self.to_addic7ed:
            return self.to_addic7ed[(alpha3, country)]
        if (alpha3,) in self.to_addic7ed:
            return self.to_addic7ed[(alpha3,)]

        return self.name_converter.convert(alpha3, country, script)

    def reverse(self, addic7ed):
        if addic7ed in self.from_addic7ed:
            return self.from_addic7ed[addic7ed]

        return self.name_converter.reverse(addic7ed)



"""
Test oldbeard.helpers

Public Methods:
    indentXML
    remove_non_release_groups
    is_media_file
    is_rar_file
    remove_file_failed
    makeDir
    searchIndexerForShowID
    list_media_files
    copyFile
    moveFile
    link
    hardlinkFile
    symlink
    moveAndSymlinkFile
    make_dirs
    rename_ep_file
    delete_empty_folders
    fileBitFilter
    chmodAsParent
    fixSetGroupID
    is_anime_in_show_list
    update_anime_support
    get_absolute_number_from_season_and_episode
    get_all_episodes_from_absolute_number
    sanitizeSceneName
    create_https_certificates
    backupVersionedFile
    restoreVersionedFile
    get_lan_ip
    check_url
    anon_url
    encrypt
    decrypt
    full_sanitizeSceneName
    get_show
    is_hidden_folder
    real_path
    is_subdirectory
    set_up_anidb_connection
    makeZip
    extractZip
    backup_config_zip
    restore_config_zip
    touchFile
    getURL
    download_file
    get_size
    generateApiKey
    remove_article
    generateCookieSecret
    verify_freespace
    pretty_time_delta
    is_file_locked
    disk_usage
    sortable_name
    manage_torrents_url
    bdecode
Private Methods:
    _check_against_names
    _setUpSession
"""

import os
import unittest
from shutil import rmtree
from zipfile import BadZipFile

import pytest

from sickchill import settings
from sickchill.helper import MEDIA_EXTENSIONS, SUBTITLE_EXTENSIONS
from sickchill.oldbeard import helpers

TEST_RESULT = "Show.Name.S01E01.HDTV.x264-SICKCHILL"
TEST_CASES = {
    "removewords": [
        TEST_RESULT,
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[cttv]",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL.RiPSaLoT",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[GloDLS]",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[EtHD]",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL-20-40",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[NO-RAR] - [ www.torrentday.com ]",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[rarbg]",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[Seedbox]",
        "{ www.SceneTime.com } - Show.Name.S01E01.HDTV.x264-SICKCHILL",
        "].[www.tensiontorrent.com] - Show.Name.S01E01.HDTV.x264-SICKCHILL",
        "[ www.TorrentDay.com ] - Show.Name.S01E01.HDTV.x264-SICKCHILL",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[silv4]",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[AndroidTwoU]",
        "[www.newpct1.com]Show.Name.S01E01.HDTV.x264-SICKCHILL",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL-NZBGEEK",
        ".www.Cpasbien.pwShow.Name.S01E01.HDTV.x264-SICKCHILL",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL [1044]",
        "[ www.Cpasbien.pw ] Show.Name.S01E01.HDTV.x264-SICKCHILL",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL.[BT]",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[vtv]",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL.[www.usabit.com]",
        "[www.Cpasbien.com] Show.Name.S01E01.HDTV.x264-SICKCHILL",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[ettv]",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[rartv]",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL-Siklopentan",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL-RP",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[PublicHD]",
        "[www.Cpasbien.pe] Show.Name.S01E01.HDTV.x264-SICKCHILL",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL[eztv]",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL-[SpastikusTV]",
        "].[ www.tensiontorrent.com ] - Show.Name.S01E01.HDTV.x264-SICKCHILL",
        "[ www.Cpasbien.com ] Show.Name.S01E01.HDTV.x264-SICKCHILL",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL- { www.SceneTime.com }",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL- [ www.torrentday.com ]",
        "Show.Name.S01E01.HDTV.x264-SICKCHILL.Renc",
    ]
}


class HelpersTests(unittest.TestCase):
    """
    Test using test generator
    """

    def __init__(self, *args, **kwargs):
        """
        Initialize test
        """
        super().__init__(*args, **kwargs)


def generator(test_strings):
    """
    Generate tests from test strings

    :param test_strings: to generate tests from
    :return: test
    """

    def _test(self):
        """
        Generate tests
        :param self:
        :return: test to run
        """
        for test_string in test_strings:
            assert helpers.remove_non_release_groups(test_string) == TEST_RESULT

    return _test


class HelpersZipTests(unittest.TestCase):
    """
    Test zip methods
    """

    def test_make_zip(self):
        """
        Test makeZip
        """
        here = os.path.dirname(__file__)
        files = [os.path.join(here, f) for f in os.listdir(here) if f[-3:] == ".py"]
        zip_path = os.path.join(here, "_test.zip")

        assert helpers.makeZip(files, zip_path)
        assert not helpers.makeZip(files, "/:/_test.zip")

        if os.path.isfile(zip_path):
            os.remove(zip_path)

    def test_extract_zip(self):
        """
        Test extractZip
        """
        here = os.path.dirname(__file__)
        files = [os.path.join(here, f) for f in os.listdir(here) if f[-3:] == ".py"]
        zip_path = os.path.join(here, "_test.zip")

        helpers.makeZip(files, zip_path)
        extract_path = os.path.join(here, "_extract_test")
        assert helpers.extractZip(zip_path, extract_path)
        assert not helpers.extractZip(zip_path, "/:/_extract")
        # Test skip directories:
        files += [os.path.join(here, "Logs")]
        helpers.makeZip(files, zip_path)
        assert helpers.extractZip(zip_path, extract_path)

        if os.path.isfile(zip_path):
            os.remove(zip_path)
        if os.path.isdir(extract_path):
            rmtree(extract_path)

    def test_backup_config_zip(self):
        """
        Test backup_config_zip
        """
        here = os.path.dirname(__file__)
        files = [os.path.join(here, f) for f in os.listdir(here) if f[-3:] in [".db", ".py"]]
        zip_path = os.path.join(here, "_backup_test.zip")

        assert helpers.backup_config_zip(files, zip_path, here)
        assert not helpers.backup_config_zip(files, "/:/_backup_test.zip")

        if os.path.isfile(zip_path):
            os.remove(zip_path)

    def test_restore_config_zip(self):
        """
        Test restore_config_zip
        """
        here = os.path.dirname(__file__)
        files = [os.path.join(here, f) for f in os.listdir(here) if f[-3:] in [".db", ".py"]]
        zip_path = os.path.join(here, "_backup_test.zip")

        helpers.backup_config_zip(files, zip_path, here)
        restore_container = os.path.join(here, "_restore_tests")
        os.path.exists(restore_container) or os.mkdir(restore_container)
        restore_path = os.path.join(restore_container, "test")
        assert not helpers.restore_config_zip(files[1], restore_path)  # test invalid zip
        assert helpers.restore_config_zip(zip_path, restore_path)
        assert helpers.restore_config_zip(zip_path, restore_path)  # test extractDir exists

        if os.path.isfile(zip_path):
            os.remove(zip_path)
        if os.path.isdir(restore_container):
            rmtree(restore_container)

    def test_is_rar_file(self):
        """
        Test is_rar_file
        """
        assert helpers.is_rar_file("lala.rar")
        assert not helpers.is_rar_file("lala.zip")
        assert not helpers.is_rar_file("lala.iso")
        assert not helpers.is_rar_file("lala.wmv")
        assert not helpers.is_rar_file("lala.avi")
        assert not helpers.is_rar_file("lala.mkv")
        assert not helpers.is_rar_file("lala.mp4")


class HelpersDirectoryTests(unittest.TestCase):
    """
    Test directory methods
    """

    @unittest.skip("Not yet implemented")
    def test_make_dirs(self):
        """
        Test make_dirs
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_delete_empty_folders(self):
        """
        Test delete_empty_folders
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_make_dir(self):
        """
        Test makeDir
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_is_hidden_folder(self):
        """
        Test is_hidden_folder
        """
        pass

    def test_real_path(self):
        """
        Test real_path
        """
        assert helpers.real_path("/usr/SickChill/../root/real/path/") == helpers.real_path("/usr/root/real/path/")

    def test_is_subdirectory(self):
        """
        Test is_subdirectory
        """
        assert helpers.is_subdirectory(subdir_path="/usr/SickChill/Downloads/Unpack", topdir_path="/usr/SickChill/Downloads")
        assert helpers.is_subdirectory(subdir_path="/usr/SickChill/Downloads/testfile.tst", topdir_path="/usr/SickChill/Downloads/")
        assert not helpers.is_subdirectory(subdir_path="/usr/SickChill/Unpack", topdir_path="/usr/SickChill/Downloads")


class HelpersFileTests(unittest.TestCase):
    """
    Test file helpers
    """

    def test_is_media_file(self):
        """
        Test is_media_file
        """
        # TODO: Add unicode tests
        # TODO: Add MAC OS resource fork tests
        # TODO: Add RARBG release tests
        # RARBG release intros should be ignored
        # MAC OS's "resource fork" files should be ignored
        # Extras should be ignored
        # and the file extension should be in the list of media extensions

        # Test all valid media extensions
        temp_name = "Show.Name.S01E01.HDTV.x264-SICKCHILL"
        extension_tests = {".".join((temp_name, ext)): True for ext in MEDIA_EXTENSIONS}
        # ...and some invalid ones
        other_extensions = ["txt", "sfv", "srr", "rar", "nfo", "zip"]
        extension_tests.update({".".join((temp_name, ext)): False for ext in other_extensions + SUBTITLE_EXTENSIONS})

        # Samples should be ignored
        sample_tests = {  # Samples should be ignored, valid samples will return False
            "Show.Name.S01E01.HDTV.sample.mkv": False,  # default case
            "Show.Name.S01E01.HDTV.sAmPle.mkv": False,  # Ignore case
            "Show.Name.S01E01.HDTV.samples.mkv": True,  # sample should not be plural
            "Show.Name.S01E01.HDTVsample.mkv": True,  # no separation, can't identify as sample
            "Sample.Show.Name.S01E01.HDTV.mkv": False,  # location doesn't matter
            "Show.Name.Sample.S01E01.HDTV.sample.mkv": False,  # location doesn't matter
            "Show.Name.S01E01.HDTV.sample1.mkv": False,  # numbered samples are ok
            "Show.Name.S01E01.HDTV.sample12.mkv": False,  # numbered samples are ok
            "Show.Name.S01E01.HDTV.sampleA.mkv": True,  # samples should not be indexed alphabetically
            "RARBG.mp4": False,
            "rarbg.MP4": False,
            "/TV/Sample.Show.Name.S01E01.HDTV-RARBG/RARBG.mp4": False,
        }

        edge_cases = {
            None: False,
            "": False,
            0: False,
            1: False,
            42: False,
            123189274981274: False,
            12.23: False,
            ("this", "is", "a tuple"): False,
        }

        for cur_test in extension_tests, sample_tests, edge_cases:
            for cur_name, expected_result in cur_test.items():
                assert helpers.is_media_file(cur_name) == expected_result, cur_name

    @unittest.skip("Not yet implemented")
    def test_is_file_locked(self):
        """
        Test is_file_locked
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_remove_file_failed(self):
        """
        Test remove_file_failed
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_list_media_files(self):
        """
        Test list_media_files
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_copy_file(self):
        """
        Test copyFile
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_move_file(self):
        """
        Test moveFile
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_rename_ep_file(self):
        """
        Test rename_ep_file
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_file_bit_filter(self):
        """
        Test fileBitFilter
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_chmod_as_parent(self):
        """
        Test chmodAsParent
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_backup_versioned_file(self):
        """
        Test backupVersionedFile
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_restore_versioned_file(self):
        """
        Test restoreVersionedFile
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_verify_free_space(self):
        """
        Test verify_freespace
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_get_disk_space_usage(self):
        """
        Test disk_usage
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_download_file(self):
        """
        Test download_file
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_get_size(self):
        """
        Test get_size
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_touch_file(self):
        """
        Test touchFile
        """
        pass


class HelpersFileLinksTests(unittest.TestCase):
    """
    Test sym and hard links
    """

    @unittest.skip("Not yet implemented")
    def test_link(self):
        """
        Test link
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_hardlink_file(self):
        """
        Test hardlinkFile
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_symlink(self):
        """
        Test symlink
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_move_and_symlink_file(self):
        """
        Test moveAndSymlinkFile
        """
        pass


class HelpersEncryptionTests(unittest.TestCase):
    """
    Test encryption and decryption
    """

    def test_create_https_certificates(self):
        """
        Test that create_https_certificates successfully generates certificate and private key
        """
        try:
            import OpenSSL
        except ImportError:
            self.skipTest("pyOpenSSL is not installed")
            return False

        base_path = os.path.dirname(__file__)
        cert_path = os.path.abspath(os.path.join(base_path, "base.crt"))
        pkey_path = os.path.abspath(os.path.join(base_path, "base.key"))

        def removeTestFiles():
            try:
                os.remove(cert_path)
                os.remove(pkey_path)
            except OSError:
                pass

        removeTestFiles()  # always remove existing
        assert helpers.create_https_certificates(cert_path, pkey_path)
        assert os.path.isfile(cert_path)
        assert os.path.isfile(pkey_path)

        FILETYPE_PEM = OpenSSL.crypto.FILETYPE_PEM
        try:
            with open(cert_path, "rb") as f:
                cert = OpenSSL.crypto.load_certificate(FILETYPE_PEM, f.read())
        except Exception:
            removeTestFiles()
            self.fail("Unable to load certificate")

        try:
            with open(pkey_path, "rb") as f:
                pkey = OpenSSL.crypto.load_privatekey(FILETYPE_PEM, f.read())
        except Exception:
            removeTestFiles()
            self.fail("Unable to load private key")

        context = OpenSSL.SSL.Context(OpenSSL.SSL.TLSv1_METHOD)
        context.use_privatekey(pkey)
        context.use_certificate(cert)
        failed = False
        try:
            context.check_privatekey()
        except OpenSSL.SSL.Error:
            failed = True
        finally:
            removeTestFiles()

        assert not failed, "private key does not match certificate"

    @unittest.skip("Not yet implemented")
    def test_encrypt(self):
        """
        Test encrypt
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_decrypt(self):
        """
        Test decrypt
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_generate_cookie_secret(self):
        """
        Test generateCookieSecret
        """
        pass


class HelpersShowTests(unittest.TestCase):
    """
    Test show methods
    """

    @unittest.skip("Not yet implemented")
    def test_search_indexer_for_show_id(self):
        """
        Test searchIndexerForShowID
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_is_anime_in_show_list(self):
        """
        Test is_anime_in_show_list
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_check_against_names(self):
        """
        Test _check_against_names
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_get_show(self):
        """
        Test get_show
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_get_abs_no_from_s_and_e(self):
        """
        Test get_absolute_number_from_season_and_episode
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_get_all_eps_from_abs_no(self):
        """
        Test get_all_episodes_from_absolute_number
        """
        pass


class HelpersConnectionTests(unittest.TestCase):
    """
    Test connections
    """

    @unittest.skip("Not yet implemented")
    def test_get_lan_ip(self):
        """
        Test get_lan_ip
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_check_url(self):
        """
        Test check_url
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_anon_url(self):
        """
        Test anon_url
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_set_up_anidb_connection(self):
        """
        Test set_up_anidb_connection
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_set_up_session(self):
        """
        Test _setUpSession
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_get_url(self):
        """
        Test getURL
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_generate_api_key(self):
        """
        Test generateApiKey
        """
        pass


class HelpersMiscTests(unittest.TestCase):
    """
    Test misc helper methods
    """

    @unittest.skip("Not yet implemented")
    def test_indent_xml(self):
        """
        Test indentXML
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_remove_non_release_groups(self):
        """
        Test remove_non_release_groups
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_fix_set_group_id(self):
        """
        Test fixSetGroupID
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_update_anime_support(self):
        """
        Test update_anime_support
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_sanitize_scene_name(self):
        """
        Test sanitizeSceneName
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_full_sanitize_scene_name(self):
        """
        Test full_sanitizeSceneName
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_remove_article(self):
        """
        Test remove_article
        """
        pass

    @unittest.skip("Not yet implemented")
    def test_pretty_time_delta(self):
        """
        Test pretty_time_delta
        """
        pass

    def test_sortable_name(self):
        """
        Test that sortable_name returns the correct show name
        """
        cases = [
            # raw_name, SORT_ARTICLE, expected
            ("The Big Bang Theory", False, "big bang theory"),
            ("A Big World", False, "big world"),
            ("An Unexpected Journey", False, "unexpected journey"),
            ("The Big Bang Theory", True, "the big bang theory"),
            ("A Big World", True, "a big world"),
            ("An Unexpected Journey", True, "an unexpected journey"),
        ]
        for raw_name, option, expected in cases:
            settings.SORT_ARTICLE = option
            assert helpers.sortable_name(raw_name) == expected

    @unittest.skip("Not yet implemented")
    def test_manage_torrents_url(self):
        """
        Test manage_torrents_url
        """
        pass


if __name__ == "__main__":
    print("==================")
    print("STARTING - Helpers TESTS")
    print("==================")
    print("######################################################################")
    for name, test_data in TEST_CASES.items():
        test_name = "test_{0}".format(name)
        test = generator(test_data)
        setattr(HelpersTests, test_name, test)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(HelpersTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(HelpersConnectionTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(HelpersDirectoryTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(HelpersEncryptionTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(HelpersFileLinksTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(HelpersFileTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(HelpersMiscTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(HelpersShowTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)

    SUITE = unittest.TestLoader().loadTestsFromTestCase(HelpersZipTests)
    unittest.TextTestRunner(verbosity=2).run(SUITE)



"""
Refiners enrich a :class:`~subliminal.video.Video` object by adding information to it.

A refiner is a simple function:

.. py:function:: refine(video, **kwargs)

    :param video: the video to refine.
    :type video: :class:`~subliminal.video.Video`
    :param \*\*kwargs: additional parameters for refiners.

"""



"""
Tests for SickChill show
"""

import unittest

from .test_coming_episodes import ComingEpisodesTests
from .test_history import HistoryTests
from .test_show import ShowTests

if __name__ == "__main__":
    print('=====> Running all test in "sickchill_tests.show" <=====')

    TEST_CLASSES = [
        ComingEpisodesTests,
        HistoryTests,
        ShowTests,
    ]

    for test_class in TEST_CLASSES:
        SUITE = unittest.TestLoader().loadTestsFromTestCase(test_class)
        unittest.TextTestRunner(verbosity=2).run(SUITE)



import socket
import sys
import threading
import zlib
from time import sleep, time

from .aniDBerrors import AniDBBannedError, AniDBError, AniDBMustAuthError, AniDBPacketCorruptedError
from .aniDBresponses import ResponseResolver


class AniDBLink(threading.Thread):
    def __init__(self, server, port, myport, logFunction, delay=2, timeout=20, log_private=False):
        super().__init__()
        self.server = server
        self.port = port
        self.target = (server, port)
        self.timeout = timeout

        self.myport = myport
        self.sock = None
        self.bound = self.connectSocket(myport, self.timeout)

        self.cmd_queue = {None: None}
        self.resp_tagged_queue = {}
        self.resp_untagged_queue = []
        self.tags = []
        self.lastpacket = time()
        self.delay = delay
        self.session = None
        self.banned = False
        self.crypt = None

        self.log = logFunction
        self.log_private = log_private

        self._stop = threading.Event()
        self._quiting = False
        self.setDaemon(True)
        self.start()

    def connectSocket(self, myport, timeout):
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.sock.settimeout(timeout)
        portlist = [myport] + [7654]
        for port in portlist:
            try:
                self.sock.bind(("", port))
            except Exception:
                continue
            else:
                self.myport = port
                return True
        else:
            return False

    def disconnectSocket(self):
        if self.sock:
            self.sock.close()

    def stop(self):
        self.log("Releasing socket and stopping link thread")
        self._quiting = True
        self.disconnectSocket()
        self._stop.set()

    def stopped(self):
        return self._stop.isSet()

    @staticmethod
    def print_log(data):
        print(data)

    def print_log_dummy(self, data):
        pass

    def run(self):
        while not self._quiting:
            try:
                data = self.sock.recv(8192)
            except socket.timeout:
                self._handle_timeouts()
                continue

            self.log("NetIO < %s" % repr(data))
            try:
                for i in range(2):
                    try:
                        tmp = data
                        resp = None
                        if tmp[:2] == b"\x00\x00":
                            self.log("Attempting inflation")
                            tmp = zlib.decompressobj().decompress(tmp[2:])
                            self.log("UnZip | %s" % repr(tmp))
                        self.log("Decoding")
                        tmp = tmp.decode("utf8")
                        self.log("Parsing")
                        resp = ResponseResolver(tmp)
                        self.log("Response success!")
                    except Exception:
                        self.log("ResponseResolver Error")
                        sys.excepthook(*sys.exc_info())
                        self.crypt = None
                        self.session = None
                    else:
                        break
                if not resp:
                    raise AniDBPacketCorruptedError("Either decrypting, decompressing or parsing the packet failed")
                cmd = self._cmd_dequeue(resp)
                resp = resp.resolve(cmd)
                resp.parse()
                if resp.rescode in ("200", "201"):
                    self.session = resp.attrs["sesskey"]
                if resp.rescode in ("209",):
                    print("sorry encryption is not supported")
                    raise Exception
                    # self.crypt=aes(md5(resp.req.apipassword+resp.attrs['salt']).digest())
                if resp.rescode in ("203", "403", "500", "501", "503", "506"):
                    self.session = None
                    self.crypt = None
                if resp.rescode in ("504", "555"):
                    self.banned = True
                    print("AniDB API informs that user or client is banned:", resp.resstr)
                resp.handle()
                if not cmd or not cmd.mode:
                    self._resp_queue(resp)
                else:
                    self.tags.remove(resp.restag)
            except Exception:
                self.log("Catastrophic error - Closing anidb thread")
                sys.excepthook(*sys.exc_info())
                print("Avoiding flood by paranoidly panicing: Aborting link thread, killing connection, releasing waiters and quiting")
                self.disconnectSocket()
                try:
                    cmd.waiter.release()
                except Exception:
                    pass

                for tag, cmd in self.cmd_queue.items():
                    try:
                        cmd.waiter.release()
                    except Exception:
                        pass

                sys.exit()

    def _handle_timeouts(self):
        willpop = []
        for tag, cmd in self.cmd_queue.items():
            if not tag:
                continue
            if time() - cmd.started > self.timeout:
                self.tags.remove(cmd.tag)
                willpop.append(cmd.tag)
                cmd.waiter.release()

        for tag in willpop:
            self.cmd_queue.pop(tag)

    def _resp_queue(self, response):
        if response.restag:
            self.resp_tagged_queue[response.restag] = response
        else:
            self.resp_untagged_queue.append(response)

    def getresponse(self, command):
        if command:
            resp = self.resp_tagged_queue.pop(command.tag)
        else:
            resp = self.resp_untagged_queue.pop()
        self.tags.remove(resp.restag)
        return resp

    def _cmd_queue(self, command):
        self.cmd_queue[command.tag] = command
        self.tags.append(command.tag)

    def _cmd_dequeue(self, resp):
        if not resp.restag:
            return None
        else:
            return self.cmd_queue.pop(resp.restag)

    def _delay(self):
        return self.delay < 2.1 and 2.1 or self.delay

    def _do_delay(self):
        age = time() - self.lastpacket
        delay = self._delay()
        if age <= delay:
            sleep(delay - age)

    def _send(self, command):
        if self.banned:
            self.log("NetIO | BANNED")
            raise AniDBBannedError("Not sending, banned")
        self._do_delay()
        self.lastpacket = time()
        command.started = time()
        data = command.raw_data()

        encoded_data = data.encode("utf-8")

        self.sock.sendto(encoded_data, self.target)

        if command.command == "AUTH" and self.log_private:
            self.log("NetIO > sensitive data is not logged!")
        else:
            self.log("NetIO > %s" % repr(data))

    def new_tag(self):
        if not len(self.tags):
            maxtag = "T000"
        else:
            maxtag = max(self.tags)
        newtag = "T%03d" % (int(maxtag[1:]) + 1)
        return newtag

    def request(self, command):
        if not self.sock and self.connectSocket(self.myport, self.timeout):
            self.log("Not connected to aniDB, not sending command")
            raise AniDBError("No Socket")

        if not (self.session and command.session) and command.command not in ("AUTH", "PING", "ENCRYPT"):
            raise AniDBMustAuthError("You must be authed to execute commands besides AUTH and PING")
        command.started = time()
        self._cmd_queue(command)
        self._send(command)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2016-5-27

@author: hustcc
'''

from datetime import datetime
from datetime import date
from datetime import time

__ALL__ = ['parse']


def parse(input):
    '''
    parse input to datetime
    '''
    if isinstance(input, datetime):
        return input
    if isinstance(input, date):
        return date_to_datetime(input)
    if isinstance(input, time):
        return time_to_datetime(input)
    if isinstance(input, (int, float)):
        return timestamp_to_datetime(input)
    if isinstance(input, (str)):
        return string_to_data_time(input)
    return None


def date_to_datetime(d):
    return _combine_date_time(d, time(0, 0, 0))


def time_to_datetime(t):
    return _combine_date_time(date.today(), t)


def timestamp_to_datetime(ts):
    '''
    parse timestamp to datetime
    '''
    return datetime.fromtimestamp(ts)


def _combine_date_time(d, t):
    if (d is not None) and (t is not None):
        return datetime(d.year, d.month, d.day, t.hour, t.minute, t.second)
    return None


def _string_to_date(date_string):
    if not date_string:
        return None
    try:
        d_array = date_string.split('-')
        return date(int(d_array[0]), int(d_array[1]), int(d_array[2]))
    except:
        return None


def _string_to_time(time_string):
    if not time_string:
        return None
    try:
        t_array = time_string.split(':')
        return time(int(t_array[0]), int(t_array[1]), int(t_array[2]))
    except:
        return None


def string_to_data_time(d):
    '''
    simple parse date string, such as:
    2016-5-27 21:22:20
    2016-05-27 21:22:2
    2016/05/27 21:22:2

    2016-05-27
    2016/5/27

    21:22:2
    '''
    if d:
        d = d.replace('/', '-')
        if ' ' in d:
            _datetime = d.split(' ')
            if len(_datetime) == 2:
                _d = _string_to_date(_datetime[0])
                _t = _string_to_time(_datetime[1])
                return _combine_date_time(_d, _t)
        else:
            # no space
            if '-' in d:
                return date_to_datetime(_string_to_date(d))
            elif ':' in d:
                return time_to_datetime(_string_to_time(d))
    return None



from collections import OrderedDict

_clients = sorted(["utorrent", "transmission", "deluge", "deluged", "download_station", "rtorrent", "qbittorrent", "mlnet", "putio"])


default_host = {
    "utorrent": "http://localhost:8000",
    "transmission": "http://localhost:9091",
    "deluge": "http://localhost:8112",
    "deluged": "scgi://localhost:58846",
    "download_station": "http://localhost:5000",
    "rtorrent": "scgi://localhost:5000",
    "qbittorrent": "http://localhost:8080",
    "mlnet": "http://localhost:4080",
    "putio": "https://api.put.io/login",
}


def getClientInstance(name):
    return __import__("sickchill.oldbeard.clients." + name.lower(), fromlist=_clients).Client


def getClientListDict(keys_only=False):
    if keys_only:
        return _clients + ["blackhole"]

    result = OrderedDict()
    result["blackhole"] = "Black Hole"
    for client in _clients:
        result[client] = getClientInstance(client)().name
    return result



"""
Redis Backends
------------------

Provides backends for talking to `Redis <http://redis.io>`_.

"""

from __future__ import absolute_import

import pickle
import warnings

from ..api import CacheBackend
from ..api import NO_VALUE

redis = None

__all__ = ("RedisBackend", "RedisSentinelBackend")


class RedisBackend(CacheBackend):
    """A `Redis <http://redis.io/>`_ backend, using the
    `redis-py <http://pypi.python.org/pypi/redis/>`_ backend.

    Example configuration::

        from dogpile.cache import make_region

        region = make_region().configure(
            'dogpile.cache.redis',
            arguments = {
                'host': 'localhost',
                'port': 6379,
                'db': 0,
                'redis_expiration_time': 60*60*2,   # 2 hours
                'distributed_lock': True,
                'thread_local_lock': False
                }
        )


    Arguments accepted in the arguments dictionary:

    :param url: string. If provided, will override separate host/port/db
     params.  The format is that accepted by ``StrictRedis.from_url()``.

     .. versionadded:: 0.4.1

    :param host: string, default is ``localhost``.

    :param password: string, default is no password.

     .. versionadded:: 0.4.1

    :param port: integer, default is ``6379``.

    :param db: integer, default is ``0``.

    :param redis_expiration_time: integer, number of seconds after setting
     a value that Redis should expire it.  This should be larger than dogpile's
     cache expiration.  By default no expiration is set.

    :param distributed_lock: boolean, when True, will use a
     redis-lock as the dogpile lock. Use this when multiple processes will be
     talking to the same redis instance. When left at False, dogpile will
     coordinate on a regular threading mutex.

    :param lock_timeout: integer, number of seconds after acquiring a lock that
     Redis should expire it.  This argument is only valid when
     ``distributed_lock`` is ``True``.

     .. versionadded:: 0.5.0

    :param socket_timeout: float, seconds for socket timeout.
     Default is None (no timeout).

     .. versionadded:: 0.5.4

    :param lock_sleep: integer, number of seconds to sleep when failed to
     acquire a lock.  This argument is only valid when
     ``distributed_lock`` is ``True``.

     .. versionadded:: 0.5.0

    :param connection_pool: ``redis.ConnectionPool`` object.  If provided,
     this object supersedes other connection arguments passed to the
     ``redis.StrictRedis`` instance, including url and/or host as well as
     socket_timeout, and will be passed to ``redis.StrictRedis`` as the
     source of connectivity.

     .. versionadded:: 0.5.4

    :param thread_local_lock: bool, whether a thread-local Redis lock object
     should be used. This is the default, but is not compatible with
     asynchronous runners, as they run in a different thread than the one
     used to create the lock.

     .. versionadded:: 0.9.1

    """

    def __init__(self, arguments):
        arguments = arguments.copy()
        self._imports()
        self.url = arguments.pop("url", None)
        self.host = arguments.pop("host", "localhost")
        self.password = arguments.pop("password", None)
        self.port = arguments.pop("port", 6379)
        self.db = arguments.pop("db", 0)
        self.distributed_lock = arguments.get("distributed_lock", False)
        self.socket_timeout = arguments.pop("socket_timeout", None)

        self.lock_timeout = arguments.get("lock_timeout", None)
        self.lock_sleep = arguments.get("lock_sleep", 0.1)
        self.thread_local_lock = arguments.get("thread_local_lock", True)

        if self.distributed_lock and self.thread_local_lock:
            warnings.warn(
                "The Redis backend thread_local_lock parameter should be "
                "set to False when distributed_lock is True"
            )

        self.redis_expiration_time = arguments.pop("redis_expiration_time", 0)
        self.connection_pool = arguments.get("connection_pool", None)
        self._create_client()

    def _imports(self):
        # defer imports until backend is used
        global redis
        import redis  # noqa

    def _create_client(self):
        if self.connection_pool is not None:
            # the connection pool already has all other connection
            # options present within, so here we disregard socket_timeout
            # and others.
            self.writer_client = redis.StrictRedis(
                connection_pool=self.connection_pool
            )
            self.reader_client = self.writer_client
        else:
            args = {}
            if self.socket_timeout:
                args["socket_timeout"] = self.socket_timeout

            if self.url is not None:
                args.update(url=self.url)
                self.writer_client = redis.StrictRedis.from_url(**args)
                self.reader_client = self.writer_client
            else:
                args.update(
                    host=self.host,
                    password=self.password,
                    port=self.port,
                    db=self.db,
                )
                self.writer_client = redis.StrictRedis(**args)
                self.reader_client = self.writer_client

    def get_mutex(self, key):
        if self.distributed_lock:
            return self.writer_client.lock(
                "_lock{0}".format(key),
                timeout=self.lock_timeout,
                sleep=self.lock_sleep,
                thread_local=self.thread_local_lock,
            )
        else:
            return None

    def get(self, key):
        value = self.reader_client.get(key)
        if value is None:
            return NO_VALUE
        return pickle.loads(value)

    def get_multi(self, keys):
        if not keys:
            return []
        values = self.reader_client.mget(keys)
        return [pickle.loads(v) if v is not None else NO_VALUE for v in values]

    def set(self, key, value):
        if self.redis_expiration_time:
            self.writer_client.setex(
                key,
                self.redis_expiration_time,
                pickle.dumps(value, pickle.HIGHEST_PROTOCOL),
            )
        else:
            self.writer_client.set(
                key, pickle.dumps(value, pickle.HIGHEST_PROTOCOL)
            )

    def set_multi(self, mapping):
        mapping = dict(
            (k, pickle.dumps(v, pickle.HIGHEST_PROTOCOL))
            for k, v in mapping.items()
        )

        if not self.redis_expiration_time:
            self.writer_client.mset(mapping)
        else:
            pipe = self.writer_client.pipeline()
            for key, value in mapping.items():
                pipe.setex(key, self.redis_expiration_time, value)
            pipe.execute()

    def delete(self, key):
        self.writer_client.delete(key)

    def delete_multi(self, keys):
        self.writer_client.delete(*keys)


class RedisSentinelBackend(RedisBackend):
    """A `Redis <http://redis.io/>`_ backend, using the
    `redis-py <http://pypi.python.org/pypi/redis/>`_ backend.
    It will use the Sentinel of a Redis cluster.

    .. versionadded:: 1.0.0

    Example configuration::

        from dogpile.cache import make_region

        region = make_region().configure(
            'dogpile.cache.redis_sentinel',
            arguments = {
                'sentinels': [
                    ['redis_sentinel_1', 26379],
                    ['redis_sentinel_2', 26379]
                ],
                'db': 0,
                'redis_expiration_time': 60*60*2,   # 2 hours
                'distributed_lock': True,
                'thread_local_lock': False
            }
        )


    Arguments accepted in the arguments dictionary:

    :param db: integer, default is ``0``.

    :param redis_expiration_time: integer, number of seconds after setting
     a value that Redis should expire it.  This should be larger than dogpile's
     cache expiration.  By default no expiration is set.

    :param distributed_lock: boolean, when True, will use a
     redis-lock as the dogpile lock. Use this when multiple processes will be
     talking to the same redis instance. When False, dogpile will
     coordinate on a regular threading mutex, Default is True.

    :param lock_timeout: integer, number of seconds after acquiring a lock that
     Redis should expire it.  This argument is only valid when
     ``distributed_lock`` is ``True``.

    :param socket_timeout: float, seconds for socket timeout.
     Default is None (no timeout).

    :param sentinels: is a list of sentinel nodes. Each node is represented by
     a pair (hostname, port).
     Default is None (not in sentinel mode).

    :param service_name: str, the service name.
     Default is 'mymaster'.

    :param sentinel_kwargs: is a dictionary of connection arguments used when
     connecting to sentinel instances. Any argument that can be passed to
     a normal Redis connection can be specified here.
     Default is {}.

    :param connection_kwargs: dict, are keyword arguments that will be used
     when establishing a connection to a Redis server.
     Default is {}.

    :param lock_sleep: integer, number of seconds to sleep when failed to
     acquire a lock.  This argument is only valid when
     ``distributed_lock`` is ``True``.

    :param thread_local_lock: bool, whether a thread-local Redis lock object
     should be used. This is the default, but is not compatible with
     asynchronous runners, as they run in a different thread than the one
     used to create the lock.

    """

    def __init__(self, arguments):
        arguments = arguments.copy()
        self._imports()
        self.password = arguments.pop("password", None)
        self.db = arguments.pop("db", 0)
        self.distributed_lock = arguments.get("distributed_lock", True)
        self.socket_timeout = arguments.pop("socket_timeout", None)
        self.sentinels = arguments.pop("sentinels", None)
        self.service_name = arguments.pop("service_name", "mymaster")
        self.sentinel_kwargs = arguments.pop("sentinel_kwargs", {})
        self.connection_kwargs = arguments.pop("connection_kwargs", {})

        self.lock_timeout = arguments.get("lock_timeout", None)
        self.lock_sleep = arguments.get("lock_sleep", 0.1)
        self.thread_local_lock = arguments.get("thread_local_lock", False)

        if self.distributed_lock and self.thread_local_lock:
            warnings.warn(
                "The Redis backend thread_local_lock parameter should be "
                "set to False when distributed_lock is True"
            )

        self.redis_expiration_time = arguments.pop("redis_expiration_time", 0)
        self._create_client()

    def _imports(self):
        # defer imports until backend is used
        global redis
        import redis.sentinel  # noqa

    def _create_client(self):
        sentinel_kwargs = {}
        sentinel_kwargs.update(self.sentinel_kwargs)
        sentinel_kwargs.setdefault("password", self.password)

        connection_kwargs = {}
        connection_kwargs.update(self.connection_kwargs)
        connection_kwargs.setdefault("password", self.password)

        if self.db is not None:
            connection_kwargs.setdefault("db", self.db)
            sentinel_kwargs.setdefault("db", self.db)
        if self.socket_timeout is not None:
            connection_kwargs.setdefault("socket_timeout", self.socket_timeout)

        sentinel = redis.sentinel.Sentinel(
            self.sentinels,
            sentinel_kwargs=sentinel_kwargs,
            **connection_kwargs
        )
        self.writer_client = sentinel.master_for(self.service_name)
        self.reader_client = sentinel.slave_for(self.service_name)



# -*- coding: utf-8 -*-

"""
This module implements the Episode functionality of TheTVDb API.
Allows to retrieve episode detailed info.

See [Episodes API section](https://api.thetvdb.com/swagger#!/Episodes)
"""

from .base import TVDB

class Episode(TVDB):
    """
    Episode class to retrieve detailed info about an episode.
    Requires the episode id.
    """
    _BASE_PATH = 'episodes'
    _URLS = {
        'info': '/{id}'
    }

    def __init__(self, id, language=''):
        """
        Initialize the episode class.

        `id` is the TheTVDb episode id. You can also provide `language`, 
        the language id you want to use to retrieve the info.
        """
        super(Episode, self).__init__(id)
        self._set_language(language)

    def info(self, language=''):
        """
        Get the episode information of the episode and set its values to the local attributes.

        You can set `language` with the language id to retrieve info in that specific language.

        It returns a dictionary with all the episode info.

        For example

            #!python
            >>> import tvdbsimple as tvdb
            >>> tvdb.KEYS.API_KEY = 'YOUR_API_KEY'
            >>> ep = tvdb.Episode(5330530)
            >>> ep.imdbId
            'tt4701544'

        """
        path = self._get_id_path('info')
        
        self._set_language(language)
        response = self._GET(path)
        self._set_attrs_to_values(response)
        return response
